@@ -240,10 +240,19 @@ void AccountSettings::doExpand()
 
 void AccountSettings::slotCustomContextMenuRequested(const QPoint &pos)
 {
+    const auto removeFolderAction = [this](QMenu *menu) {
+        return menu->addAction(tr("Remove folder sync connection"), this, &AccountSettings::slotRemoveCurrentFolder);
+    };
     QTreeView *tv = ui->_folderList;
     QModelIndex index = tv->indexAt(pos);
     if (!index.isValid()) {
         return;
+    } else if (!_model->data(index, FolderStatusDelegate::IsReady).toBool()) {
+        QMenu *menu = new QMenu(tv);
+        menu->setAttribute(Qt::WA_DeleteOnClose);
+        removeFolderAction(menu);
+        menu->popup(QCursor::pos());
+        return;
     }
 
     if (_model->classify(index) == FolderStatusModel::SubFolder) {
@@ -270,7 +279,7 @@ void AccountSettings::slotCustomContextMenuRequested(const QPoint &pos)
         });
 
 
-        menu->popup(tv->mapToGlobal(pos));
+        menu->popup(QCursor::pos());
         return;
     }
 
@@ -284,7 +293,7 @@ void AccountSettings::slotCustomContextMenuRequested(const QPoint &pos)
     bool folderConnected = _model->data(index, FolderStatusDelegate::FolderAccountConnected).toBool();
     auto folderMan = FolderMan::instance();
     QPointer<Folder> folder = folderMan->folder(alias);
-    if (!folder)
+    if (!folder || !OC_ENSURE(folder->isReady()))
         return;
 
     QMenu *menu = new QMenu(tv);
@@ -312,30 +321,10 @@ void AccountSettings::slotCustomContextMenuRequested(const QPoint &pos)
     ac = menu->addAction(folderPaused ? tr("Resume sync") : tr("Pause sync"));
     connect(ac, &QAction::triggered, this, &AccountSettings::slotEnableCurrentFolder);
 
-    ac = menu->addAction(tr("Remove folder sync connection"));
-    connect(ac, &QAction::triggered, this, &AccountSettings::slotRemoveCurrentFolder);
+    removeFolderAction(menu);
 
-    if (folder->virtualFilesEnabled()) {
-        auto availabilityMenu = menu->addMenu(tr("Availability"));
-        auto availability = folder->vfs().availability(QString());
-        if (availability) {
-            ac = availabilityMenu->addAction(Utility::vfsCurrentAvailabilityText(*availability));
-            ac->setEnabled(false);
-        }
-
-        ac = availabilityMenu->addAction(Utility::vfsPinActionText());
-        ac->setEnabled(!availability || *availability != VfsItemAvailability::AlwaysLocal);
-        connect(ac, &QAction::triggered, this, [this]() { slotSetCurrentFolderAvailability(PinState::AlwaysLocal); });
-
-        ac = availabilityMenu->addAction(Utility::vfsFreeSpaceActionText());
-        ac->setEnabled(!availability
-                || !(*availability == VfsItemAvailability::OnlineOnly
-                    || *availability == VfsItemAvailability::AllDehydrated));
-        connect(ac, &QAction::triggered, this, [this]() { slotSetCurrentFolderAvailability(PinState::OnlineOnly); });
-        if (!Theme::instance()->forceVirtualFilesOption()) {
-            ac = menu->addAction(tr("Disable virtual file support..."));
-            connect(ac, &QAction::triggered, this, &AccountSettings::slotDisableVfsCurrentFolder);
-        }
+    if (folder->virtualFilesEnabled() && !Theme::instance()->forceVirtualFilesOption()) {
+        menu->addAction(tr("Disable virtual file support..."), this, &AccountSettings::slotDisableVfsCurrentFolder);
     }
 
     if (Theme::instance()->showVirtualFilesOption()
@@ -348,7 +337,7 @@ void AccountSettings::slotCustomContextMenuRequested(const QPoint &pos)
     }
 
 
-    menu->popup(tv->mapToGlobal(pos));
+    menu->popup(QCursor::pos());
 }
 
 void AccountSettings::slotFolderListClicked(const QModelIndex &indx)
@@ -711,7 +700,7 @@ void AccountSettings::slotEnableCurrentFolder(bool terminate)
         currentlyPaused = f->syncPaused();
         if (!currentlyPaused && !terminate) {
             // check if a sync is still running and if so, ask if we should terminate.
-            if (f->isBusy()) { // its still running
+            if (f->isSyncRunning()) { // its still running
                 auto msgbox = new QMessageBox(QMessageBox::Question, tr("Sync Running"),
                     tr("The syncing operation is running.<br/>Do you want to terminate it?"),
                     QMessageBox::Yes | QMessageBox::No, this);
@@ -727,9 +716,10 @@ void AccountSettings::slotEnableCurrentFolder(bool terminate)
 
         // message box can return at any time while the thread keeps running,
         // so better check again after the user has responded.
-        if (f->isBusy() && terminate) {
+        if (f->isSyncRunning() && terminate) {
             f->slotTerminateSync();
         }
+        f->slotNextSyncFullLocalDiscovery(); // ensure we don't forget about local errors
         f->setSyncPaused(!currentlyPaused);
 
         // keep state for the icon setting.
@@ -772,7 +762,7 @@ void AccountSettings::slotForceSyncCurrentFolder()
         }
 
         selectedFolder->slotWipeErrorBlacklist(); // issue #6757
-
+        selectedFolder->slotNextSyncFullLocalDiscovery(); // ensure we don't forget about local errors
         // Insert the selected folder at the front of the queue
         folderMan->scheduleFolderNext(selectedFolder);
     }
@@ -962,7 +952,7 @@ void AccountSettings::refreshSelectiveSyncStatus()
     QString msg;
     int cnt = 0;
     for (Folder *folder : FolderMan::instance()->map()) {
-        if (folder->accountState() != _accountState) {
+        if (folder->accountState() != _accountState || !folder->isReady()) {
             continue;
         }
 
