@@ -28,6 +28,15 @@
 #include <QTimer>
 #include <qfontmetrics.h>
 
+namespace {
+
+inline const QLatin1String userExplicitlySignedOutC()
+{
+    return QLatin1String("userExplicitlySignedOut");
+}
+
+} // anonymous namespace
+
 namespace OCC {
 
 Q_LOGGING_CATEGORY(lcAccountState, "gui.account.state", QtInfoMsg)
@@ -112,14 +121,25 @@ AccountState::~AccountState()
 {
 }
 
-AccountState *AccountState::loadFromSettings(AccountPtr account, QSettings & /*settings*/)
+AccountState *AccountState::loadFromSettings(AccountPtr account, const QSettings &settings)
 {
     auto accountState = new AccountState(account);
+    const bool userExplicitlySignedOut = settings.value(userExplicitlySignedOutC(), false).toBool();
+    if (userExplicitlySignedOut) {
+        // see writeToSettings below
+        accountState->_state = SignedOut;
+    }
     return accountState;
 }
 
-void AccountState::writeToSettings(QSettings & /*settings*/)
+void AccountState::writeToSettings(QSettings &settings) const
 {
+    // The SignedOut state is the only state where the client should *not* ask for credentials, nor
+    // try to connect to the server. All other states should transition to Connected by either
+    // (re-)trying to make a connection, or by authenticating (AskCredentials). So we save the
+    // SignedOut state to indicate that the client should not try to re-connect the next time it
+    // is started.
+    settings.setValue(userExplicitlySignedOutC(), _state == SignedOut);
 }
 
 AccountPtr AccountState::account() const
@@ -392,17 +412,20 @@ void AccountState::slotConnectionValidatorResult(ConnectionValidator::Status sta
 void AccountState::slotInvalidCredentials()
 {
     if (!_waitingForNewCredentials) {
-        qCInfo(lcAccountState) << "Invalid credentials for" << _account->url().toString()
-                               << "asking user";
+        qCInfo(lcAccountState) << "Invalid credentials for" << _account->url().toString();
 
         _waitingForNewCredentials = true;
         if (account()->credentials()->ready()) {
             account()->credentials()->invalidateToken();
         }
         if (auto creds = qobject_cast<HttpCredentials *>(account()->credentials())) {
-            if (creds->refreshAccessToken())
+            qCInfo(lcAccountState) << "refreshing oauth";
+            if (creds->refreshAccessToken()) {
                 return;
+            }
+            qCInfo(lcAccountState) << "refreshing oauth failed";
         }
+        qCInfo(lcAccountState) << "asking user";
         account()->credentials()->askFromUser();
     }
     setState(AskingCredentials);
