@@ -15,8 +15,8 @@
 #include <QtGui>
 #include <QtWidgets>
 
-#include "activitylistmodel.h"
 #include "activitywidget.h"
+#include "configfile.h"
 #include "syncresult.h"
 #include "logger.h"
 #include "theme.h"
@@ -28,7 +28,6 @@
 #include "account.h"
 #include "accountstate.h"
 #include "accountmanager.h"
-#include "activityitemdelegate.h"
 #include "protocolwidget.h"
 #include "issueswidget.h"
 #include "QProgressIndicator.h"
@@ -38,6 +37,10 @@
 #include "theme.h"
 #include "ocsjob.h"
 
+#include "models/activitylistmodel.h"
+#include "models/expandingheaderview.h"
+#include "models/models.h"
+
 #include "ui_activitywidget.h"
 
 #include <climits>
@@ -55,17 +58,20 @@ ActivityWidget::ActivityWidget(QWidget *parent)
 {
     _ui->setupUi(this);
 
-// Adjust copyToClipboard() when making changes here!
-#if defined(Q_OS_MAC)
-    _ui->_activityList->setMinimumWidth(400);
-#endif
-
     _model = new ActivityListModel(this);
-    ActivityItemDelegate *delegate = new ActivityItemDelegate;
-    delegate->setParent(this);
-    _ui->_activityList->setItemDelegate(delegate);
-    _ui->_activityList->setAlternatingRowColors(true);
-    _ui->_activityList->setModel(_model);
+    _sortModel = new QSortFilterProxyModel(this);
+    _sortModel->setSourceModel(_model);
+    _ui->_activityList->setModel(_sortModel);
+    _sortModel->setSortRole(Models::UnderlyingDataRole);
+    auto header = new ExpandingHeaderView(QStringLiteral("ActivityListModelHeader"), _ui->_activityList);
+    _ui->_activityList->setHorizontalHeader(header);
+    header->hideSection(static_cast<int>(ActivityListModel::ActivityRole::Path));
+    header->setSectionResizeMode(QHeaderView::Interactive);
+    header->setSortIndicator(static_cast<int>(ActivityListModel::ActivityRole::PointInTime), Qt::DescendingOrder);
+
+    connect(_ui->_filterButton, &QAbstractButton::clicked, this, [this] {
+        ProtocolWidget::showFilterMenu(_ui->_filterButton, _sortModel);
+    });
 
     _ui->_notifyLabel->hide();
     _ui->_notifyScroll->hide();
@@ -89,20 +95,20 @@ ActivityWidget::ActivityWidget(QWidget *parent)
             showLabels();
         }
 
-        for (auto it = _widgetForNotifId.cbegin(); it != _widgetForNotifId.cend(); ++it) {
-            if (it.key().second == ast->account()->displayName()) {
-                scheduleWidgetToRemove(it.value());
+        for (const auto widget : qAsConst(_widgetForNotifId)) {
+            if (widget->activity().uuid() == ast->account()->uuid()) {
+                scheduleWidgetToRemove(widget);
             }
         }
     });
 
-    _copyBtn = _ui->_dialogButtonBox->addButton(tr("Copy"), QDialogButtonBox::ActionRole);
-    _copyBtn->setToolTip(tr("Copy the activity list to the clipboard."));
-    connect(_copyBtn, &QAbstractButton::clicked, this, &ActivityWidget::copyToClipboard);
-
-    connect(_model, &QAbstractItemModel::rowsInserted, this, &ActivityWidget::rowsInserted);
-
-    connect(_ui->_activityList, &QListView::activated, this, &ActivityWidget::slotOpenFile);
+    connect(_model, &QAbstractItemModel::modelReset, this, &ActivityWidget::dataChanged);
+    connect(_ui->_activityList, &QListView::customContextMenuRequested, this, &ActivityWidget::slotItemContextMenu);
+    header->setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(header, &QListView::customContextMenuRequested, header, [header, this] {
+        auto menu = ProtocolWidget::showFilterMenu(header, _sortModel);
+        header->addResetActionToMenu(menu);
+    });
 
     connect(&_removeTimer, &QTimer::timeout, this, &ActivityWidget::slotCheckToCleanWidgets);
     _removeTimer.setInterval(1000);
@@ -170,55 +176,6 @@ void ActivityWidget::slotAccountActivityStatus(AccountState *ast, int statusCode
     showLabels();
 }
 
-// FIXME: Reused from protocol widget. Move over to utilities.
-QString ActivityWidget::timeString(QDateTime dt, QLocale::FormatType format) const
-{
-    const QLocale loc = QLocale::system();
-    QString dtFormat = loc.dateTimeFormat(format);
-    static const QRegExp re("(HH|H|hh|h):mm(?!:s)");
-    dtFormat.replace(re, "\\1:mm:ss");
-    return loc.toString(dt, dtFormat);
-}
-
-void ActivityWidget::storeActivityList(QTextStream &ts)
-{
-    ActivityList activities = _model->activityList();
-
-    foreach (Activity activity, activities) {
-        ts << right
-           // account name
-           << qSetFieldWidth(30)
-           << activity._accName
-           // separator
-           << qSetFieldWidth(0) << ","
-
-           // date and time
-           << qSetFieldWidth(34)
-           << activity._dateTime.toString()
-           // separator
-           << qSetFieldWidth(0) << ","
-
-           // file
-           << qSetFieldWidth(30)
-           << activity._file
-           // separator
-           << qSetFieldWidth(0) << ","
-
-           // subject
-           << qSetFieldWidth(100)
-           << activity._subject
-           // separator
-           << qSetFieldWidth(0) << ","
-
-           // message (mostly empty)
-           << qSetFieldWidth(55)
-           << activity._message
-           //
-           << qSetFieldWidth(0)
-           << endl;
-    }
-}
-
 void ActivityWidget::checkActivityTabVisibility()
 {
     int accountCount = AccountManager::instance()->accounts().count();
@@ -235,31 +192,22 @@ void ActivityWidget::checkActivityTabVisibility()
     emit hideActivityTab(!hasAccountsWithActivity && !hasNotifications);
 }
 
-void ActivityWidget::slotOpenFile(QModelIndex indx)
-{
-    qCDebug(lcActivity) << indx.isValid() << indx.data(ActivityItemDelegate::PathRole).toString() << QFile::exists(indx.data(ActivityItemDelegate::PathRole).toString());
-    if (indx.isValid()) {
-        QString fullPath = indx.data(ActivityItemDelegate::PathRole).toString();
-
-        if (QFile::exists(fullPath)) {
-            showInFileManager(fullPath);
-        }
-    }
-}
-
 // GUI: Display the notifications.
 // All notifications in list are coming from the same account
 // but in the _widgetForNotifId hash widgets for all accounts are
 // collected.
 void ActivityWidget::slotBuildNotificationDisplay(const ActivityList &list)
 {
+    if (list.empty()) {
+        return;
+    }
+    // compute the count to display later
     QHash<QString, int> accNotified;
-    QString listAccountName;
 
     // Whether a new notification widget was added to the notificationLayout.
     bool newNotificationShown = false;
 
-    foreach (auto activity, list) {
+    for (const auto &activity : list) {
         if (_blacklistedNotifications.contains(activity)) {
             qCInfo(lcActivity) << "Activity in blacklist, skip";
             continue;
@@ -267,8 +215,8 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList &list)
 
         NotificationWidget *widget = nullptr;
 
-        if (_widgetForNotifId.contains(activity.ident())) {
-            widget = _widgetForNotifId[activity.ident()];
+        if (_widgetForNotifId.contains(activity.id())) {
+            widget = _widgetForNotifId[activity.id()];
         } else {
             widget = new NotificationWidget(this);
             connect(widget, &NotificationWidget::sendNotificationRequest,
@@ -279,15 +227,12 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList &list)
             _notificationsLayout->addWidget(widget);
             // _ui->_notifyScroll->setMinimumHeight( widget->height());
             _ui->_notifyScroll->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContentsOnFirstShow);
-            _widgetForNotifId[activity.ident()] = widget;
+            _widgetForNotifId[activity.id()] = widget;
             newNotificationShown = true;
         }
 
         widget->setActivity(activity);
 
-        // remember the list account name for the strayCat handling below.
-        listAccountName = activity._accName;
-
         // handle gui logs. In order to NOT annoy the user with every fetching of the
         // notifications the notification id is stored in a Set. Only if an id
         // is not in the set, it qualifies for guiLog.
@@ -298,35 +243,28 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList &list)
         if (_guiLogTimer.elapsed() > 60 * 60 * 1000) {
             _guiLoggedNotifications.clear();
         }
-        if (!_guiLoggedNotifications.contains(activity._id)) {
-            QString host = activity._accName;
-            // store the name of the account that sends the notification to be
+        if (!_guiLoggedNotifications.contains(activity.id())) {
+            // store the uui of the account that sends the notification to be
             // able to add it to the tray notification
-            if (!host.isEmpty()) {
-                if (accNotified.contains(host)) {
-                    accNotified[host] = accNotified[host] + 1;
-                } else {
-                    accNotified[host] = 1;
-                }
-            }
-            _guiLoggedNotifications.insert(activity._id);
+            accNotified[activity.accName()]++;
+            _guiLoggedNotifications.insert(activity.id());
         }
     }
 
     // check if there are widgets that have no corresponding activity from
     // the server any more. Collect them in a list
-    QList<Activity::Identifier> strayCats;
-    foreach (auto id, _widgetForNotifId.keys()) {
-        NotificationWidget *widget = _widgetForNotifId[id];
 
+    const auto accId = list.first().uuid();
+    QList<Activity::Identifier> strayCats;
+    for (auto it = _widgetForNotifId.cbegin(); it != _widgetForNotifId.cend(); ++it) {
         bool found = false;
         // do not mark widgets of other accounts to delete.
-        if (widget->activity()._accName != listAccountName) {
+        if (it.value()->activity().uuid() != accId) {
             continue;
         }
 
-        foreach (auto activity, list) {
-            if (activity.ident() == id) {
+        for (const auto &activity : list) {
+            if (activity.id() == it.key()) {
                 // found an activity
                 found = true;
                 break;
@@ -334,12 +272,12 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList &list)
         }
         if (!found) {
             // the activity does not exist any more.
-            strayCats.append(id);
+            strayCats.append(it.key());
         }
     }
 
     // .. and now delete all these stray cat widgets.
-    foreach (auto strayCatId, strayCats) {
+    for (const auto &strayCatId : strayCats) {
         NotificationWidget *widgetToGo = _widgetForNotifId[strayCatId];
         scheduleWidgetToRemove(widgetToGo, 0);
     }
@@ -380,10 +318,10 @@ void ActivityWidget::slotSendNotificationRequest(const QString &accountName, con
     qCInfo(lcActivity) << "Server Notification Request " << verb << link << "on account" << accountName;
     NotificationWidget *theSender = qobject_cast<NotificationWidget *>(sender());
 
-    const QStringList validVerbs = QStringList() << "GET"
-                                                 << "PUT"
-                                                 << "POST"
-                                                 << "DELETE";
+    const QStringList validVerbs = { QStringLiteral("GET"),
+        QStringLiteral("PUT"),
+        QStringLiteral("POST"),
+        QStringLiteral("DELETE") };
 
     if (validVerbs.contains(verb)) {
         AccountStatePtr acc = AccountManager::instance()->account(accountName);
@@ -454,7 +392,7 @@ void ActivityWidget::slotRequestCleanupAndBlacklist(const Activity &blacklistAct
         _blacklistedNotifications.append(blacklistActivity);
     }
 
-    NotificationWidget *widget = _widgetForNotifId[blacklistActivity.ident()];
+    NotificationWidget *widget = _widgetForNotifId[blacklistActivity.id()];
     scheduleWidgetToRemove(widget);
 }
 
@@ -486,7 +424,7 @@ void ActivityWidget::slotCheckToCleanWidgets()
 
         if (currentTime > t) {
             // found one to remove!
-            Activity::Identifier id = widget->activity().ident();
+            Activity::Identifier id = widget->activity().id();
             _widgetForNotifId.remove(id);
             widget->deleteLater();
             it = _widgetsToRemove.erase(it);
@@ -506,6 +444,31 @@ void ActivityWidget::slotCheckToCleanWidgets()
     }
 }
 
+void ActivityWidget::slotItemContextMenu()
+{
+    auto rows = _ui->_activityList->selectionModel()->selectedRows();
+    auto menu = new QMenu(this);
+    menu->setAttribute(Qt::WA_DeleteOnClose);
+
+    // keep in sync with ProtocolWidget::showContextMenu
+    menu->addAction(tr("Copy to clipboard"), this, [text = Models::formatSelection(rows, Models::UnderlyingDataRole)] {
+        QApplication::clipboard()->setText(text);
+    });
+
+    if (rows.size() == 1) {
+        // keep in sync with ProtocolWidget::showContextMenu
+        const auto localPath = rows.first().siblingAtColumn(static_cast<int>(ActivityListModel::ActivityRole::Path)).data(Models::UnderlyingDataRole).toString();
+        if (!localPath.isEmpty()) {
+            menu->addAction(tr("Show in file browser"), this, [localPath] {
+                if (QFileInfo::exists(localPath)) {
+                    showInFileManager(localPath);
+                }
+            });
+        }
+    }
+    menu->popup(QCursor::pos());
+}
+
 
 /* ==================================================================== */
 
@@ -520,22 +483,18 @@ ActivitySettings::ActivitySettings(QWidget *parent)
     hbox->addWidget(_tab);
     _activityWidget = new ActivityWidget(this);
     _activityTabId = _tab->addTab(_activityWidget, Theme::instance()->applicationIcon(), tr("Server Activity"));
-    connect(_activityWidget, &ActivityWidget::copyToClipboard, this, &ActivitySettings::slotCopyToClipboard);
     connect(_activityWidget, &ActivityWidget::hideActivityTab, this, &ActivitySettings::setActivityTabHidden);
     connect(_activityWidget, &ActivityWidget::guiLog, this, &ActivitySettings::guiLog);
     connect(_activityWidget, &ActivityWidget::newNotification, this, &ActivitySettings::slotShowActivityTab);
 
     _protocolWidget = new ProtocolWidget(this);
     _protocolTabId = _tab->addTab(_protocolWidget, Theme::instance()->syncStateIcon(SyncResult::Success), tr("Sync Protocol"));
-    connect(_protocolWidget, &ProtocolWidget::copyToClipboard, this, &ActivitySettings::slotCopyToClipboard);
 
     _issuesWidget = new IssuesWidget(this);
     _syncIssueTabId = _tab->addTab(_issuesWidget, Theme::instance()->syncStateIcon(SyncResult::Problem), QString());
     slotShowIssueItemCount(0); // to display the label.
     connect(_issuesWidget, &IssuesWidget::issueCountUpdated,
         this, &ActivitySettings::slotShowIssueItemCount);
-    connect(_issuesWidget, &IssuesWidget::copyToClipboard,
-        this, &ActivitySettings::slotCopyToClipboard);
 
     // Add a progress indicator to spin if the acitivity list is updated.
     _progressIndicator = new QProgressIndicator(this);
@@ -545,7 +504,7 @@ ActivitySettings::ActivitySettings(QWidget *parent)
         this, &ActivitySettings::slotRegularNotificationCheck);
 
     // connect a model signal to stop the animation.
-    connect(_activityWidget, &ActivityWidget::rowsInserted, _progressIndicator, &QProgressIndicator::stopAnimation);
+    connect(_activityWidget, &ActivityWidget::dataChanged, _progressIndicator, &QProgressIndicator::stopAnimation);
 
     // We want the protocol be the default
     _tab->setCurrentIndex(1);
@@ -590,39 +549,11 @@ void ActivitySettings::slotShowActivityTab()
     }
 }
 
-void ActivitySettings::slotShowIssuesTab(const QString &folderAlias)
+void ActivitySettings::slotShowIssuesTab()
 {
     if (_syncIssueTabId == -1)
         return;
     _tab->setCurrentIndex(_syncIssueTabId);
-
-    _issuesWidget->showFolderErrors(folderAlias);
-}
-
-void ActivitySettings::slotCopyToClipboard()
-{
-    QString text;
-    QTextStream ts(&text);
-
-    int idx = _tab->currentIndex();
-    QString message;
-
-    if (idx == _activityTabId) {
-        // the activity widget
-        _activityWidget->storeActivityList(ts);
-        message = tr("The server activity list has been copied to the clipboard.");
-    } else if (idx == _protocolTabId) {
-        // the protocol widget
-        _protocolWidget->storeSyncActivity(ts);
-        message = tr("The sync activity list has been copied to the clipboard.");
-    } else if (idx == _syncIssueTabId) {
-        // issues Widget
-        message = tr("The list of unsynced items has been copied to the clipboard.");
-        _issuesWidget->storeSyncIssues(ts);
-    }
-
-    QApplication::clipboard()->setText(text);
-    emit guiLog(tr("Copied to clipboard"), message);
 }
 
 void ActivitySettings::slotRemoveAccount(AccountState *ptr)
@@ -655,8 +586,7 @@ void ActivitySettings::slotRefresh(AccountState *ptr)
 
 void ActivitySettings::slotRegularNotificationCheck()
 {
-    AccountManager *am = AccountManager::instance();
-    foreach (AccountStatePtr a, am->accounts()) {
+    for (const auto &a : AccountManager::instance()->accounts()) {
         slotRefresh(a.data());
     }
 }
