@@ -47,7 +47,21 @@
 #include <QPushButton>
 #include <QApplication>
 
-static const char versionC[] = "version";
+namespace {
+
+/*
+ * [Accounts]
+ * 1\Folders\4\version=2
+ * 1\FoldersWithPlaceholders\3\version=3
+ */
+auto versionC()
+{
+    return QStringLiteral("version");
+}
+
+constexpr int WinVfsSettingsVersion = 4;
+constexpr int SettingsVersion = 2;
+}
 
 namespace OCC {
 
@@ -77,10 +91,13 @@ Folder::Folder(const FolderDefinition &definition,
     _syncResult.setStatus(status);
 
     // check if the local path exists
-    checkLocalPath();
+    const auto folderOk = checkLocalPath();
 
     _syncResult.setFolder(_definition.alias);
 
+    // those errors should not persist over sessions
+    _journal.wipeErrorBlacklistCategory(SyncJournalErrorBlacklistRecord::Category::LocalSoftError);
+
     _engine.reset(new SyncEngine(_accountState->account(), path(), remotePath(), &_journal));
     // pass the setting if hidden files are to be ignored, will be read in csync_update
     _engine->setIgnoreHiddenFiles(_definition.ignoreHiddenFiles);
@@ -141,8 +158,10 @@ Folder::Folder(const FolderDefinition &definition,
         saveToSettings();
     }
 
-    // Initialize the vfs plugin
-    startVfs();
+    if (folderOk) {
+        // Initialize the vfs plugin
+        startVfs();
+    }
 }
 
 Folder::~Folder()
@@ -155,8 +174,11 @@ Folder::~Folder()
     _engine.reset();
 }
 
-void Folder::checkLocalPath()
+bool Folder::checkLocalPath()
 {
+#ifdef Q_OS_WIN
+    Utility::NtfsPermissionLookupRAII ntfs_perm;
+#endif
     const QFileInfo fi(_definition.localPath);
     _canonicalLocalPath = fi.canonicalFilePath();
 #ifdef Q_OS_MAC
@@ -170,21 +192,27 @@ void Folder::checkLocalPath()
         _canonicalLocalPath.append('/');
     }
 
-    if (fi.isDir() && fi.isReadable()) {
+    if (fi.isDir() && fi.isReadable() && fi.isWritable()) {
         qCDebug(lcFolder) << "Checked local path ok";
     } else {
+        QString error;
         // Check directory again
         if (!FileSystem::fileExists(_definition.localPath, fi)) {
-            _syncResult.appendErrorString(tr("Local folder %1 does not exist.").arg(_definition.localPath));
-            _syncResult.setStatus(SyncResult::SetupError);
+            error = tr("Local folder %1 does not exist.").arg(_definition.localPath);
         } else if (!fi.isDir()) {
-            _syncResult.appendErrorString(tr("%1 should be a folder but is not.").arg(_definition.localPath));
-            _syncResult.setStatus(SyncResult::SetupError);
+            error = tr("%1 should be a folder but is not.").arg(_definition.localPath);
         } else if (!fi.isReadable()) {
-            _syncResult.appendErrorString(tr("%1 is not readable.").arg(_definition.localPath));
+            error = tr("%1 is not readable.").arg(_definition.localPath);
+        } else if (!fi.isWritable()) {
+            error = tr("%1 is not writable.").arg(_definition.localPath);
+        }
+        if (!error.isEmpty()) {
+            _syncResult.appendErrorString(error);
             _syncResult.setStatus(SyncResult::SetupError);
+            return false;
         }
     }
+    return true;
 }
 
 QString Folder::shortGuiRemotePathOrAppName() const
@@ -248,11 +276,6 @@ QString Folder::cleanPath() const
     return cleanedPath;
 }
 
-bool Folder::isBusy() const
-{
-    return isSyncRunning();
-}
-
 bool Folder::isSyncRunning() const
 {
     return _engine->isSyncRunning() || _vfs->isHydrating();
@@ -283,13 +306,18 @@ bool Folder::syncPaused() const
 
 bool Folder::canSync() const
 {
-    return !syncPaused() && accountState()->isConnected();
+    return !syncPaused() && accountState()->isConnected() && isReady();
+}
+
+bool Folder::isReady() const
+{
+    return _vfsIsReady;
 }
 
 bool Folder::dueToSync() const
 {
     // conditions taken from previous folderman implementation
-    if (isSyncRunning() || etagJob() || isBusy() || !canSync()) {
+    if (isSyncRunning() || etagJob() || !canSync()) {
         return false;
     }
 
@@ -495,12 +523,20 @@ void Folder::startVfs()
     OC_ENFORCE(_vfs);
     OC_ENFORCE(_vfs->mode() == _definition.virtualFilesMode);
 
+    const auto result = Vfs::checkAvailability(path(), _vfs->mode());
+    if (!result) {
+        _syncResult.appendErrorString(result.error());
+        _syncResult.setStatus(SyncResult::SetupError);
+        return;
+    }
+
     VfsSetupParams vfsParams;
     vfsParams.filesystemPath = path();
     vfsParams.remotePath = remotePathTrailingSlash();
     vfsParams.account = _accountState->account();
     vfsParams.journal = &_journal;
-    vfsParams.providerName = Theme::instance()->appNameGUI();
+    vfsParams.providerDisplayName = Theme::instance()->appNameGUI();
+    vfsParams.providerName = Theme::instance()->appName();
     vfsParams.providerVersion = Theme::instance()->version();
     vfsParams.multipleAccountsRegistered = AccountManager::instance()->accounts().size() > 1;
 
@@ -510,14 +546,23 @@ void Folder::startVfs()
     connect(&_engine->syncFileStatusTracker(), &SyncFileStatusTracker::fileStatusChanged,
             _vfs.data(), &Vfs::fileStatusChanged);
 
-    _vfs->start(vfsParams);
 
-    // Immediately mark the sqlite temporaries as excluded. They get recreated
-    // on db-open and need to get marked again every time.
-    QString stateDbFile = _journal.databaseFilePath();
-    _journal.open();
-    _vfs->fileStatusChanged(stateDbFile + "-wal", SyncFileStatus::StatusExcluded);
-    _vfs->fileStatusChanged(stateDbFile + "-shm", SyncFileStatus::StatusExcluded);
+    connect(_vfs.data(), &Vfs::started, this, [this] {
+        // Immediately mark the sqlite temporaries as excluded. They get recreated
+        // on db-open and need to get marked again every time.
+        QString stateDbFile = _journal.databaseFilePath();
+        _journal.open();
+        _vfs->fileStatusChanged(stateDbFile + QStringLiteral("-wal"), SyncFileStatus::StatusExcluded);
+        _vfs->fileStatusChanged(stateDbFile + QStringLiteral("-shm"), SyncFileStatus::StatusExcluded);
+        _vfsIsReady = true;
+    });
+    connect(_vfs.data(), &Vfs::error, this, [this](const QString &error) {
+        _syncResult.appendErrorString(error);
+        _syncResult.setStatus(SyncResult::SetupError);
+        _vfsIsReady = false;
+    });
+
+    _vfs->start(vfsParams);
 }
 
 int Folder::slotDiscardDownloadProgress()
@@ -557,13 +602,31 @@ void Folder::slotWatchedPathChanged(const QString &path, ChangeReason reason)
         return;
     }
 
-    auto relativePath = path.midRef(this->path().size());
+    const QString relativePath = path.mid(this->path().size());
+    if (reason == ChangeReason::UnLock) {
+        journalDb()->wipeErrorBlacklistEntry(relativePath, SyncJournalErrorBlacklistRecord::Category::LocalSoftError);
+
+        {
+            // horrible hack to compensate that we don't handle folder deletes on a per file basis
+            int index = 0;
+            QString p = relativePath;
+            while ((index = p.lastIndexOf(QLatin1Char('/'))) != -1) {
+                p = p.left(index);
+                const auto rec = journalDb()->errorBlacklistEntry(p);
+                if (rec.isValid()) {
+                    if (rec._errorCategory == SyncJournalErrorBlacklistRecord::Category::LocalSoftError) {
+                        journalDb()->wipeErrorBlacklistEntry(p);
+                    }
+                }
+            }
+        }
+    }
 
     // Add to list of locally modified paths
     //
     // We do this before checking for our own sync-related changes to make
     // extra sure to not miss relevant changes.
-    auto relativePathBytes = relativePath.toUtf8();
+    const auto relativePathBytes = relativePath.toUtf8();
     _localDiscoveryTracker->addTouchedPath(relativePathBytes);
 
 // The folder watcher fires a lot of bogus notifications during
@@ -592,7 +655,7 @@ void Folder::slotWatchedPathChanged(const QString &path, ChangeReason reason)
             && !FileSystem::fileChanged(path, record._fileSize, record._modtime)) {
             spurious = true;
 
-            if (auto pinState = _vfs->pinState(relativePath.toString())) {
+            if (auto pinState = _vfs->pinState(relativePath)) {
                 if (*pinState == PinState::AlwaysLocal && record.isVirtualFile())
                     spurious = false;
                 if (*pinState == PinState::OnlineOnly && record.isFile())
@@ -765,6 +828,13 @@ void Folder::slotTerminateSync()
 
 void Folder::wipeForRemoval()
 {
+    // prevent interaction with the db etc
+    _vfsIsReady = false;
+
+    // stop reacting to changes
+    // especially the upcoming deletion of the db
+    _folderWatcher.reset();
+
     // Delete files that have been partially downloaded.
     slotDiscardDownloadProgress();
 
@@ -773,12 +843,12 @@ void Folder::wipeForRemoval()
     _journal.close(); // close the sync journal
 
     // Remove db and temporaries
-    QString stateDbFile = _engine->journal()->databaseFilePath();
+    const QString stateDbFile = _engine->journal()->databaseFilePath();
 
     QFile file(stateDbFile);
     if (file.exists()) {
         if (!file.remove()) {
-            qCWarning(lcFolder) << "Failed to remove existing csync StateDB " << stateDbFile;
+            qCCritical(lcFolder) << "Failed to remove existing csync StateDB " << stateDbFile;
         } else {
             qCInfo(lcFolder) << "wipe: Removed csync StateDB " << stateDbFile;
         }
@@ -802,11 +872,12 @@ bool Folder::reloadExcludes()
     return _engine->excludedFiles().reloadExcludeFiles();
 }
 
-void Folder::startSync(const QStringList &pathList)
+void Folder::startSync()
 {
-    Q_UNUSED(pathList)
+    Q_ASSERT(isReady());
+    Q_ASSERT(_folderWatcher);
 
-    if (isBusy()) {
+    if (!OC_ENSURE(!isSyncRunning())) {
         qCCritical(lcFolder) << "ERROR csync is still running and new sync requested.";
         return;
     }
@@ -1105,7 +1176,7 @@ void Folder::slotFolderConflicts(const QString &folder, const QStringList &confl
         r.setNumOldConflictItems(conflictPaths.size() - r.numNewConflictItems());
 }
 
-void Folder::warnOnNewExcludedItem(const SyncJournalFileRecord &record, const QStringRef &path)
+void Folder::warnOnNewExcludedItem(const SyncJournalFileRecord &record, QStringView path)
 {
     // Never warn for items in the database
     if (record.isValid())
@@ -1189,9 +1260,10 @@ void Folder::setSaveBackwardsCompatible(bool save)
 
 void Folder::registerFolderWatcher()
 {
-    if (_folderWatcher)
+    if (!isReady()) {
         return;
-    if (!QDir(path()).exists())
+    }
+    if (_folderWatcher)
         return;
 
     _folderWatcher.reset(new FolderWatcher(this));
@@ -1261,11 +1333,9 @@ void FolderDefinition::save(QSettings &settings, const FolderDefinition &folder)
     settings.setValue(QStringLiteral("virtualFilesMode"), Vfs::modeToString(folder.virtualFilesMode));
 
     // Ensure new vfs modes won't be attempted by older clients
-    if (folder.virtualFilesMode == Vfs::WindowsCfApi) {
-        settings.setValue(QLatin1String(versionC), 3);
-    } else {
-        settings.setValue(QLatin1String(versionC), 2);
-    }
+    const int version = folder.virtualFilesMode == Vfs::WindowsCfApi ? WinVfsSettingsVersion : SettingsVersion;
+    Q_ASSERT(version <= maxSettingsVersion());
+    settings.setValue(versionC(), version);
 
     // Happens only on Windows when the explorer integration is enabled.
     if (!folder.navigationPaneClsid.isNull())
@@ -1338,9 +1408,4 @@ QString FolderDefinition::absoluteJournalPath() const
     return QDir(localPath).filePath(journalPath);
 }
 
-QString FolderDefinition::defaultJournalPath(AccountPtr account)
-{
-    return SyncJournalDb::makeDbName(localPath, QStringLiteral("journal"));
-}
-
 } // namespace OCC
