@@ -374,6 +374,9 @@ public slots:
      */
     void schedulePathForLocalDiscovery(const QString &relativePath);
 
+    /// Reloads the excludes, used when changing the user-defined excludes after saving them to disk.
+    bool reloadExcludes();
+
 private slots:
     void slotSyncStarted();
     void slotSyncFinished(bool);
@@ -407,7 +410,7 @@ private slots:
     void slotFolderConflicts(const QString &folder, const QStringList &conflictPaths);
 
     /** Warn users if they create a file or folder that is selective-sync excluded */
-    void warnOnNewExcludedItem(const SyncJournalFileRecord &record, const QStringRef &path);
+    void warnOnNewExcludedItem(const SyncJournalFileRecord &record, QStringView path);
 
     /** Warn users about an unreliable folder watcher */
     void slotWatcherUnreliable(const QString &message);
@@ -425,8 +428,6 @@ private slots:
 private:
     void connectSyncRoot();
 
-    bool reloadExcludes();
-
     void showSyncResultPopup();
 
     bool checkLocalPath();
