@@ -13,20 +13,23 @@
  */
 
 #include "discovery.h"
+#include "common/checksums.h"
 #include "common/syncjournaldb.h"
+#include "csync.h"
+#include "csync_exclude.h"
+#include "owncloudpropagator.h"
+#include "syncengine.h"
 #include "syncfileitem.h"
-#include <QDebug>
+#include "vio/csync_vio_local.h"
+
 #include <algorithm>
 #include <set>
-#include <QTextCodec>
-#include "vio/csync_vio_local.h"
-#include <QFileInfo>
-#include <QFile>
-#include <QThreadPool>
-#include "common/checksums.h"
-#include "csync_exclude.h"
-#include "csync_util.h"
 
+#include <QDebug>
+#include <QFile>
+#include <QFileInfo>
+#include <QTextCodec>
+#include <QThreadPool>
 
 namespace OCC {
 
@@ -87,8 +90,9 @@ void ProcessDirectoryJob::process()
     auto pathU8 = _currentFolder._original.toUtf8();
     if (!_discoveryData->_statedb->listFilesInPath(pathU8, [&](const SyncJournalFileRecord &rec) {
             auto name = pathU8.isEmpty() ? QString::fromUtf8(rec._path) : QString::fromUtf8(rec._path.constData() + (pathU8.size() + 1));
-            if (rec.isVirtualFile() && isVfsWithSuffix())
-                chopVirtualFileSuffix(name);
+            if (rec.isVirtualFile() && isVfsWithSuffix()) {
+                name = chopVirtualFileSuffix(name);
+            }
             auto &dbEntry = entries[name].dbEntry;
             dbEntry = rec;
             setupDbPinStateActions(dbEntry);
@@ -112,8 +116,7 @@ void ProcessDirectoryJob::process()
             auto &suffixedEntry = entries[e.name];
             bool hasOtherData = suffixedEntry.serverEntry.isValid() || suffixedEntry.dbEntry.isValid();
 
-            auto nonvirtualName = e.name;
-            chopVirtualFileSuffix(nonvirtualName);
+            auto nonvirtualName = chopVirtualFileSuffix(e.name);
             auto &nonvirtualEntry = entries[nonvirtualName];
             // If the non-suffixed entry has no data, move it
             if (!nonvirtualEntry.localEntry.isValid()) {
@@ -180,7 +183,7 @@ void ProcessDirectoryJob::process()
 
 bool ProcessDirectoryJob::handleExcluded(const QString &path, const QString &localName, bool isDirectory, bool isHidden, bool isSymlink)
 {
-    auto excluded = _discoveryData->_excludes->traversalPatternMatch(path, isDirectory ? ItemTypeDirectory : ItemTypeFile);
+    auto excluded = _discoveryData->_excludes->traversalPatternMatch(&path, isDirectory ? ItemTypeDirectory : ItemTypeFile);
 
     // FIXME: move to ExcludedFiles 's regexp ?
     bool isInvalidPattern = false;
@@ -219,6 +222,9 @@ bool ProcessDirectoryJob::handleExcluded(const QString &path, const QString &loc
     } else if (excluded == CSYNC_FILE_SILENTLY_EXCLUDED || excluded == CSYNC_FILE_EXCLUDE_AND_REMOVE) {
         emit _discoveryData->silentlyExcluded(path);
         return true;
+    } else if (excluded == CSYNC_FILE_EXCLUDE_RESERVED) {
+        emit _discoveryData->excluded(path, excluded);
+        return true;
     }
 
     auto item = SyncFileItemPtr::create();
@@ -234,29 +240,37 @@ bool ProcessDirectoryJob::handleExcluded(const QString &path, const QString &loc
         case CSYNC_NOT_EXCLUDED:
         case CSYNC_FILE_SILENTLY_EXCLUDED:
         case CSYNC_FILE_EXCLUDE_AND_REMOVE:
+        case CSYNC_FILE_EXCLUDE_RESERVED:
             qFatal("These were handled earlier");
         case CSYNC_FILE_EXCLUDE_LIST:
             item->_errorString = tr("File is listed on the ignore list.");
+            item->_status = SyncFileItem::Excluded;
             break;
         case CSYNC_FILE_EXCLUDE_INVALID_CHAR:
             if (item->_file.endsWith(QLatin1Char('.'))) {
                 item->_errorString = tr("File names ending with a period are not supported on this file system.");
             } else {
-                char invalid = '\0';
-                for (const char x : QByteArrayLiteral("\\:?*\"<>|")) {
-                    if (item->_file.contains(QLatin1Char(x))) {
-                        invalid = x;
-                        break;
+                const auto unsupportedCharacter = [](const QString &fName) {
+                    const auto unsupportedCharacter = QStringLiteral("\\:?*\"<>|");
+                    for (const auto &x : unsupportedCharacter) {
+                        if (fName.contains(x)) {
+                            return x;
+                        }
                     }
-                }
-                if (invalid) {
+                    return QChar();
+                }(item->_file);
+
+                if (!unsupportedCharacter.isNull()) {
                     item->_errorString = tr("File names containing the character '%1' are not supported on this file system.")
-                                             .arg(QLatin1Char(invalid));
-                }
-                if (isInvalidPattern) {
+                                             .arg(unsupportedCharacter);
+                } else if (isInvalidPattern) {
                     item->_errorString = tr("File name contains at least one invalid character");
                 } else {
                     item->_errorString = tr("The file name is a reserved name on this file system.");
+                    if (!localName.isEmpty()) {
+                        // The file is indeed a system file and that we don't upload it is no problem
+                        item->_status = SyncFileItem::Excluded;
+                    }
                 }
             }
             break;
@@ -268,6 +282,7 @@ bool ProcessDirectoryJob::handleExcluded(const QString &path, const QString &loc
             break;
         case CSYNC_FILE_EXCLUDE_HIDDEN:
             item->_errorString = tr("File/Folder is ignored because it's hidden.");
+            item->_status = SyncFileItem::Excluded;
             break;
         case CSYNC_FILE_EXCLUDE_STAT_FAILED:
             item->_errorString = tr("Stat failed.");
@@ -328,18 +343,6 @@ void ProcessDirectoryJob::processFile(PathTuple path,
     if (item->_type == ItemTypeVirtualFileDehydration)
         item->_type = ItemTypeFile;
 
-    // VFS suffixed files on the server are ignored
-    if (isVfsWithSuffix()) {
-        if (hasVirtualFileSuffix(serverEntry.name)
-            || (localEntry.isVirtualFile && !dbEntry.isVirtualFile() && hasVirtualFileSuffix(QString::fromUtf8(dbEntry._path)))) {
-            item->_instruction = CSYNC_INSTRUCTION_IGNORE;
-            item->_errorString = tr("File has extension reserved for virtual files.");
-            _childIgnored = true;
-            emit _discoveryData->itemDiscovered(item);
-            return;
-        }
-    }
-
     if (serverEntry.isValid()) {
         processFileAnalyzeRemoteInfo(item, path, localEntry, serverEntry, dbEntry);
         return;
@@ -460,15 +463,14 @@ void ProcessDirectoryJob::processFileAnalyzeRemoteInfo(
     item->_modtime = serverEntry.modtime;
     item->_size = serverEntry.size;
 
-    auto postProcessServerNew = [=] () {
-        auto tmp_path = path;
+    auto postProcessServerNew = [=]() mutable {
         if (item->isDirectory()) {
             _pendingAsyncJobs++;
-            _discoveryData->checkSelectiveSyncNewFolder(tmp_path._server, serverEntry.remotePerm,
+            _discoveryData->checkSelectiveSyncNewFolder(path._server, serverEntry.remotePerm,
                 [=](bool result) {
                     --_pendingAsyncJobs;
                     if (!result) {
-                        processFileAnalyzeLocalInfo(item, tmp_path, localEntry, serverEntry, dbEntry, _queryServer);
+                        processFileAnalyzeLocalInfo(item, path, localEntry, serverEntry, dbEntry, _queryServer);
                     }
                     QTimer::singleShot(0, _discoveryData, &DiscoveryPhase::scheduleMoreJobs);
                 });
@@ -482,9 +484,9 @@ void ProcessDirectoryJob::processFileAnalyzeRemoteInfo(
             && _pinState != PinState::AlwaysLocal) {
             item->_type = ItemTypeVirtualFile;
             if (isVfsWithSuffix())
-                addVirtualFileSuffix(tmp_path._original);
+                addVirtualFileSuffix(path._original);
         }
-        processFileAnalyzeLocalInfo(item, tmp_path, localEntry, serverEntry, dbEntry, _queryServer);
+        processFileAnalyzeLocalInfo(item, path, localEntry, serverEntry, dbEntry, _queryServer);
     };
 
     // Potential NEW/NEW conflict is handled in AnalyzeLocal
@@ -594,8 +596,7 @@ void ProcessDirectoryJob::processFileAnalyzeRemoteInfo(
             // we need to make a request to the server to know that the original file is deleted on the server
             _pendingAsyncJobs++;
             auto job = new RequestEtagJob(_discoveryData->_account, _discoveryData->_remoteFolder + originalPath, this);
-            connect(job, &RequestEtagJob::finishedWithResult, this, [=](const HttpResult<QString> &etag) {
-                auto tmp_path = path;
+            connect(job, &RequestEtagJob::finishedWithResult, this, [=](const HttpResult<QByteArray> &etag) mutable {
                 _pendingAsyncJobs--;
                 QTimer::singleShot(0, _discoveryData, &DiscoveryPhase::scheduleMoreJobs);
                 if (etag || etag.error().code != 404 ||
@@ -611,8 +612,8 @@ void ProcessDirectoryJob::processFileAnalyzeRemoteInfo(
                 // In case the deleted item was discovered in parallel
                 _discoveryData->findAndCancelDeletedJob(originalPath);
 
-                postProcessRename(tmp_path);
-                processFileFinalize(item, tmp_path, item->isDirectory(), item->_instruction == CSYNC_INSTRUCTION_RENAME ? NormalQuery : ParentDontExist, _queryServer);
+                postProcessRename(path);
+                processFileFinalize(item, path, item->isDirectory(), item->_instruction == CSYNC_INSTRUCTION_RENAME ? NormalQuery : ParentDontExist, _queryServer);
             });
             job->start();
             done = true; // Ideally, if the origin still exist on the server, we should continue searching...  but that'd be difficult
@@ -989,23 +990,22 @@ void ProcessDirectoryJob::processFileAnalyzeLocalInfo(
         // We must query the server to know if the etag has not changed
         _pendingAsyncJobs++;
         QString serverOriginalPath = _discoveryData->_remoteFolder + _discoveryData->adjustRenamedPath(originalPath, SyncFileItem::Down);
-        if (base.isVirtualFile() && isVfsWithSuffix())
-            chopVirtualFileSuffix(serverOriginalPath);
+        if (base.isVirtualFile() && isVfsWithSuffix()) {
+            serverOriginalPath = chopVirtualFileSuffix(serverOriginalPath);
+        }
         auto job = new RequestEtagJob(_discoveryData->_account, serverOriginalPath, this);
-        connect(job, &RequestEtagJob::finishedWithResult, this, [=](const HttpResult<QString> &etag) {
-            auto tmp_path = path;
-            auto tmp_recurseQueryServer = recurseQueryServer;
-            if (!etag || (etag->toUtf8() != base._etag && !item->isDirectory()) || _discoveryData->isRenamed(originalPath)) {
+        connect(job, &RequestEtagJob::finishedWithResult, this, [=](const HttpResult<QByteArray> &etag) mutable {
+            if (!etag || (etag.get() != base._etag && !item->isDirectory()) || _discoveryData->isRenamed(originalPath)) {
                 qCInfo(lcDisco) << "Can't rename because the etag has changed or the directory is gone" << originalPath;
                 // Can't be a rename, leave it as a new.
                 postProcessLocalNew();
             } else {
                 // In case the deleted item was discovered in parallel
                 _discoveryData->findAndCancelDeletedJob(originalPath);
-                processRename(tmp_path);
-                tmp_recurseQueryServer = etag->toUtf8() == base._etag ? ParentNotChanged : NormalQuery;
+                processRename(path);
+                recurseQueryServer = etag.get() == base._etag ? ParentNotChanged : NormalQuery;
             }
-            processFileFinalize(item, tmp_path, item->isDirectory(), NormalQuery, tmp_recurseQueryServer);
+            processFileFinalize(item, path, item->isDirectory(), NormalQuery, recurseQueryServer);
             _pendingAsyncJobs--;
             QTimer::singleShot(0, _discoveryData, &DiscoveryPhase::scheduleMoreJobs);
         });
@@ -1059,25 +1059,22 @@ void ProcessDirectoryJob::processFileConflict(const SyncFileItemPtr &item, Proce
     // we recieved the etag (Issue #5106)
     auto up = _discoveryData->_statedb->getUploadInfo(path._original);
     if (up._valid && up._contentChecksum == serverEntry.checksumHeader) {
-        // Solve the conflict into an upload, or nothing
+        // Solve the conflict into an upload, or update meta data
         item->_instruction = up._modtime == localEntry.modtime && up._size == localEntry.size
-            ? CSYNC_INSTRUCTION_NONE : CSYNC_INSTRUCTION_SYNC;
+            ? CSYNC_INSTRUCTION_UPDATE_METADATA
+            : CSYNC_INSTRUCTION_SYNC;
         item->_direction = SyncFileItem::Up;
 
-        // Update the etag and other server metadata in the journal already
-        // (We can't use a typical CSYNC_INSTRUCTION_UPDATE_METADATA because
-        // we must not store the size/modtime from the file system)
-        OCC::SyncJournalFileRecord rec;
-        if (_discoveryData->_statedb->getFileRecord(path._original, &rec)) {
-            rec._path = path._original.toUtf8();
-            rec._etag = serverEntry.etag;
-            rec._fileId = serverEntry.fileId;
-            rec._modtime = serverEntry.modtime;
-            rec._type = item->_type;
-            rec._fileSize = serverEntry.size;
-            rec._remotePerm = serverEntry.remotePerm;
-            rec._checksumHeader = serverEntry.checksumHeader;
-            _discoveryData->_statedb->setFileRecord(rec);
+        if (item->_instruction == CSYNC_INSTRUCTION_UPDATE_METADATA) {
+            // Update the etag and other server metadata in the journal already
+            Q_ASSERT(item->_file == path._original);
+            Q_ASSERT(item->_size == serverEntry.size);
+            Q_ASSERT(item->_modtime == serverEntry.modtime);
+            Q_ASSERT(!serverEntry.etag.isEmpty());
+            item->_etag = serverEntry.etag;
+            item->_fileId = serverEntry.fileId;
+            item->_remotePerm = serverEntry.remotePerm;
+            item->_checksumHeader = serverEntry.checksumHeader;
         }
         return;
     }
@@ -1099,8 +1096,7 @@ void ProcessDirectoryJob::processFileFinalize(
                 addVirtualFileSuffix(item->_renameTarget);
             else
                 addVirtualFileSuffix(item->_file);
-        }
-        if (item->_type == ItemTypeVirtualFileDehydration
+        } else if (item->_type == ItemTypeVirtualFileDehydration
             && item->_instruction == CSYNC_INSTRUCTION_SYNC) {
             if (item->_renameTarget.isEmpty()) {
                 item->_renameTarget = item->_file;
@@ -1343,7 +1339,7 @@ int ProcessDirectoryJob::processSubJobs(int nbJobs)
     }
 
     int started = 0;
-    foreach (auto *rj, _runningJobs) {
+    for (auto *rj : qAsConst(_runningJobs)) {
         started += rj->processSubJobs(nbJobs - started);
         if (started >= nbJobs)
             return started;
@@ -1361,7 +1357,7 @@ int ProcessDirectoryJob::processSubJobs(int nbJobs)
 
 void ProcessDirectoryJob::dbError()
 {
-    _discoveryData->fatalError(tr("Error while reading the database"));
+    Q_EMIT _discoveryData->fatalError(tr("Error while reading the database"));
 }
 
 void ProcessDirectoryJob::addVirtualFileSuffix(QString &str) const
@@ -1376,14 +1372,13 @@ bool ProcessDirectoryJob::hasVirtualFileSuffix(const QString &str) const
     return str.endsWith(_discoveryData->_syncOptions._vfs->fileSuffix());
 }
 
-void ProcessDirectoryJob::chopVirtualFileSuffix(QString &str) const
+QString ProcessDirectoryJob::chopVirtualFileSuffix(const QString &str) const
 {
     if (!isVfsWithSuffix())
-        return;
-    bool hasSuffix = hasVirtualFileSuffix(str);
-    OC_ASSERT(hasSuffix);
-    if (hasSuffix)
-        str.chop(_discoveryData->_syncOptions._vfs->fileSuffix().size());
+        return str;
+    // ensure we do it only with virtual files in this class
+    Q_ASSERT(hasVirtualFileSuffix(str));
+    return _discoveryData->_syncOptions._vfs->underlyingFileName(str);
 }
 
 DiscoverySingleDirectoryJob *ProcessDirectoryJob::startAsyncServerQuery()
