@@ -83,6 +83,7 @@ bool FileSystem::setModTime(const QString &filename, time_t modTime)
     if (rc != 0) {
         qCWarning(lcFileSystem) << "Error setting mtime for" << filename
                                 << "failed: rc" << rc << ", errno:" << errno;
+        Q_ASSERT(false);
         return false;
     }
     return true;
@@ -137,47 +138,44 @@ qint64 FileSystem::getSize(const QString &filename)
 }
 
 // Code inspired from Qt5's QDir::removeRecursively
-bool FileSystem::removeRecursively(const QString &path, const std::function<void(const QString &path, bool isDir)> &onDeleted, QStringList *errors)
+bool FileSystem::removeRecursively(const QString &path,
+    RemoveEntryList *success,
+    RemoveEntryList *locked,
+    RemoveErrorList *errors)
 {
     bool allRemoved = true;
     QDirIterator di(path, QDir::AllEntries | QDir::Hidden | QDir::System | QDir::NoDotAndDotDot);
 
+    QString removeError;
     while (di.hasNext()) {
         di.next();
         const QFileInfo &fi = di.fileInfo();
-        bool removeOk = false;
         // The use of isSymLink here is okay:
         // we never want to go into this branch for .lnk files
-        bool isDir = fi.isDir() && !fi.isSymLink() && !FileSystem::isJunction(fi.absoluteFilePath());
+        const bool isDir = fi.isDir() && !fi.isSymLink() && !FileSystem::isJunction(fi.absoluteFilePath());
         if (isDir) {
-            removeOk = removeRecursively(path + QLatin1Char('/') + di.fileName(), onDeleted, errors); // recursive
+            allRemoved &= removeRecursively(path + QLatin1Char('/') + di.fileName(), success, locked, errors); // recursive
         } else {
-            QString removeError;
-            removeOk = FileSystem::remove(di.filePath(), &removeError);
-            if (removeOk) {
-                if (onDeleted)
-                    onDeleted(di.filePath(), false);
+            if (FileSystem::isFileLocked(di.filePath(), FileSystem::LockMode::Exclusive)) {
+                locked->push_back({ di.filePath(), isDir });
+                allRemoved = false;
             } else {
-                if (errors) {
-                    errors->append(QCoreApplication::translate("FileSystem", "Error removing '%1': %2")
-                                       .arg(QDir::toNativeSeparators(di.filePath()), removeError));
+                if (FileSystem::remove(di.filePath(), &removeError)) {
+                    success->push_back({ di.filePath(), isDir });
+                } else {
+                    errors->push_back({ { di.filePath(), isDir }, removeError });
+                    qCWarning(lcFileSystem) << "Error removing " << di.filePath() << ':' << removeError;
+                    allRemoved = false;
                 }
-                qCWarning(lcFileSystem) << "Error removing " << di.filePath() << ':' << removeError;
             }
         }
-        if (!removeOk)
-            allRemoved = false;
     }
     if (allRemoved) {
         allRemoved = QDir().rmdir(path);
         if (allRemoved) {
-            if (onDeleted)
-                onDeleted(path, true);
+            success->push_back({ path, true });
         } else {
-            if (errors) {
-                errors->append(QCoreApplication::translate("FileSystem", "Could not remove folder '%1'")
-                                   .arg(QDir::toNativeSeparators(path)));
-            }
+            errors->push_back({ { path, true }, QCoreApplication::translate("FileSystem", "Could not remove folder") });
             qCWarning(lcFileSystem) << "Error removing folder" << path;
         }
     }
