@@ -16,6 +16,7 @@
 
 #include "config.h"
 
+#include <QCoreApplication>
 #include <QDir>
 #include <QStringList>
 #include <QtGlobal>
@@ -30,50 +31,48 @@
 #ifdef Q_OS_WIN
 #include <io.h> // for stdout
 #include <fcntl.h>
+#include <comdef.h>
 #endif
 
-namespace OCC {
+namespace {
+constexpr int CrashLogSize = 20;
 
-static void mirallLogCatcher(QtMsgType type, const QMessageLogContext &ctx, const QString &message)
+#ifdef Q_OS_WIN
+bool isDebuggerPresent()
 {
-    auto logger = Logger::instance();
-    if (!logger->isNoop()) {
-        logger->doLog(qFormatLogMessage(type, ctx, message));
-    }
-    if(type == QtCriticalMsg || type == QtFatalMsg) {
-        std::cerr << qPrintable(qFormatLogMessage(type, ctx, message)) << std::endl;
-    }
-
-    if(type == QtFatalMsg) {
-        if (!logger->isNoop()) {
-            logger->close();
-        }
-#if defined(Q_OS_WIN)
-    // Make application terminate in a way that can be caught by the crash reporter
-        Utility::crash();
-#endif
+    BOOL debugged;
+    if (!CheckRemoteDebuggerPresent(GetCurrentProcess(), &debugged)) {
+        const auto error = GetLastError();
+        qDebug() << "Failed to detect debugger" << QString::fromWCharArray(_com_error(error).ErrorMessage());
     }
+    return debugged;
 }
-
+#endif
+}
+namespace OCC {
 
 Logger *Logger::instance()
 {
-    static Logger log;
-    return &log;
+    static auto *log = [] {
+        auto log = new Logger;
+        qAddPostRoutine([] {
+            Logger::instance()->close();
+            delete Logger::instance();
+        });
+        return log;
+    }();
+    return log;
 }
 
 Logger::Logger(QObject *parent)
     : QObject(parent)
-    , _showTime(true)
-    , _doFileFlush(false)
-    , _logExpire(0)
-    , _logDebug(false)
 {
-    qSetMessagePattern(QStringLiteral("%{time MM-dd hh:mm:ss:zzz} [ %{type} %{category} ]%{if-debug}\t[ %{function} ]%{endif}:\t%{message}"));
+    qSetMessagePattern(loggerPattern());
+    _crashLog.resize(CrashLogSize);
 #ifndef NO_MSG_HANDLER
-    qInstallMessageHandler(mirallLogCatcher);
-#else
-    Q_UNUSED(mirallLogCatcher)
+    qInstallMessageHandler([](QtMsgType type, const QMessageLogContext &ctx, const QString &message) {
+            Logger::instance()->doLog(type, ctx, message);
+        });
 #endif
 }
 
@@ -84,27 +83,9 @@ Logger::~Logger()
 #endif
 }
 
-void Logger::log(Log log)
+QString Logger::loggerPattern()
 {
-    QString msg;
-    if (_showTime) {
-        msg = log.timeStamp.toString(QStringLiteral("MM-dd hh:mm:ss:zzz")) + QLatin1Char(' ');
-    }
-
-    msg += log.message;
-    // _logs.append(log);
-    // std::cout << qPrintable(log.message) << std::endl;
-
-    doLog(msg);
-}
-
-/**
- * Returns true if doLog does nothing and need not to be called
- */
-bool Logger::isNoop() const
-{
-    QMutexLocker lock(&_mutex);
-    return !_logstream;
+    return QStringLiteral("%{time MM-dd hh:mm:ss:zzz} [ %{type} %{category} ]%{if-debug}\t[ %{function} ]%{endif}:\t%{message}");
 }
 
 bool Logger::isLoggingToFile() const
@@ -113,21 +94,36 @@ bool Logger::isLoggingToFile() const
     return _logstream;
 }
 
-void Logger::doLog(const QString &msg)
+void Logger::doLog(QtMsgType type, const QMessageLogContext &ctx, const QString &message)
 {
+    const QString msg = qFormatLogMessage(type, ctx, message) + QLatin1Char('\n');
     {
         QMutexLocker lock(&_mutex);
+        _crashLogIndex = (_crashLogIndex + 1) % CrashLogSize;
+        _crashLog[_crashLogIndex] = msg;
         if (_logstream) {
-            (*_logstream) << msg << endl;
+            (*_logstream) << msg;
             if (_doFileFlush)
                 _logstream->flush();
         }
+#if defined(Q_OS_WIN)
+        if (isDebuggerPresent()) {
+            OutputDebugStringW(reinterpret_cast<const wchar_t *>(msg.utf16()));
+        }
+#endif
+        if (type == QtFatalMsg) {
+            dumpCrashLog();
+            close();
+#if defined(Q_OS_WIN)
+            // Make application terminate in a way that can be caught by the crash reporter
+            Utility::crash();
+#endif
+        }
     }
 }
 
 void Logger::close()
 {
-    QMutexLocker lock(&_mutex);
     if (_logstream)
     {
         _logstream->flush();
@@ -150,16 +146,7 @@ void Logger::setLogFile(const QString &name)
 
     bool openSucceeded = false;
     if (name == QLatin1String("-")) {
-#ifdef Q_OS_WIN
-        if (AttachConsole(ATTACH_PARENT_PROCESS)) {
-            // atache to the parent console output, if its an interactive terminal
-            CONSOLE_SCREEN_BUFFER_INFO csbi;
-            if (GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), &csbi)) {
-                freopen("CONOUT$", "w", stdout);
-                freopen("CONOUT$", "w", stderr);
-            }
-        }
-#endif
+        attacheToConsole();
         openSucceeded = _logFile.open(stdout, QIODevice::WriteOnly);
     } else {
         _logFile.setFileName(name);
@@ -232,7 +219,26 @@ void Logger::setLogRules(const QSet<QString> &rules)
 {
     static const QString defaultRule = qEnvironmentVariable("QT_LOGGING_RULES").replace(QLatin1Char(';'), QLatin1Char('\n'));
     _logRules = rules;
-    QLoggingCategory::setFilterRules(rules.toList().join(QLatin1Char('\n')) + QLatin1Char('\n') + defaultRule);
+    QString tmp;
+    QTextStream out(&tmp);
+    for (const auto &p : rules) {
+        out << p << QLatin1Char('\n');
+    }
+    out << defaultRule;
+    qDebug() << tmp;
+    QLoggingCategory::setFilterRules(tmp);
+}
+
+void Logger::dumpCrashLog()
+{
+    QFile logFile(QDir::tempPath() + QStringLiteral("/" APPLICATION_NAME "-crash.log"));
+    if (logFile.open(QFile::WriteOnly)) {
+        QTextStream out(&logFile);
+        out.setCodec("UTF-8");
+        for (int i = 1; i <= CrashLogSize; ++i) {
+            out << _crashLog[(_crashLogIndex + i) % CrashLogSize];
+        }
+    }
 }
 
 static bool compressLog(const QString &originalName, const QString &targetName)
@@ -275,11 +281,11 @@ void Logger::enterNextLogFile()
         QString newLogName = now.toString(QStringLiteral("yyyyMMdd_HHmm")) + QStringLiteral("_owncloud.log");
 
         // Expire old log files and deal with conflicts
-        QStringList files = dir.entryList(QStringList(QStringLiteral("*owncloud.log.*")),
+        const auto &files = dir.entryList(QStringList(QStringLiteral("*owncloud.log.*")),
             QDir::Files, QDir::Name);
         QRegExp rx(QStringLiteral(R"(.*owncloud\.log\.(\d+).*)"));
         int maxNumber = -1;
-        foreach (const QString &s, files) {
+        for (const auto &s : files) {
             if (_logExpire.count() > 0) {
                 std::chrono::seconds expireSeconds(_logExpire);
                 QFileInfo fileInfo(dir.absoluteFilePath(s));
@@ -312,4 +318,22 @@ void Logger::enterNextLogFile()
     }
 }
 
+void OCC::Logger::attacheToConsole()
+{
+    if (_consoleIsAttached) {
+        return;
+    }
+    _consoleIsAttached = true;
+#ifdef Q_OS_WIN
+    if (!isDebuggerPresent() && AttachConsole(ATTACH_PARENT_PROCESS)) {
+        // atache to the parent console output, if its an interactive terminal
+        CONSOLE_SCREEN_BUFFER_INFO csbi;
+        if (GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), &csbi)) {
+            freopen("CONOUT$", "w", stdout);
+            freopen("CONOUT$", "w", stderr);
+        }
+    }
+#endif
+}
+
 } // namespace OCC
