@@ -14,24 +14,24 @@
  */
 
 #include "owncloudpropagator.h"
-#include "common/syncjournaldb.h"
-#include "common/syncjournalfilerecord.h"
-#include "propagatedownload.h"
-#include "propagateupload.h"
-#include "propagateuploadtus.h"
-#include "propagateremotedelete.h"
-#include "propagateremotemove.h"
-#include "propagateremotemkdir.h"
-#include "propagatorjobs.h"
-#include "filesystem.h"
-#include "common/utility.h"
 #include "account.h"
 #include "common/asserts.h"
+#include "common/syncjournaldb.h"
+#include "common/syncjournalfilerecord.h"
+#include "common/utility.h"
 #include "discoveryphase.h"
+#include "filesystem.h"
+#include "propagatedownload.h"
+#include "propagateremotedelete.h"
+#include "propagateremotemkdir.h"
+#include "propagateremotemove.h"
+#include "propagateupload.h"
+#include "propagateuploadtus.h"
+#include "propagatorjobs.h"
 
 #ifdef Q_OS_WIN
-#include <windef.h>
 #include <winbase.h>
+#include <windef.h>
 #endif
 
 #include <QApplication>
@@ -173,13 +173,11 @@ static SyncJournalErrorBlacklistRecord createBlacklistEntry(
 
     entry._ignoreDuration = qBound(minBlacklistTime, entry._ignoreDuration, maxBlacklistTime);
 
-    if (item._status == SyncFileItem::SoftError) {
-        // Track these errors, but don't actively suppress them.
-        entry._ignoreDuration = 0;
-    }
-
     if (item._httpErrorCode == 507) {
-        entry._errorCategory = SyncJournalErrorBlacklistRecord::InsufficientRemoteStorage;
+        entry._errorCategory = SyncJournalErrorBlacklistRecord::Category::InsufficientRemoteStorage;
+    } else if (item._httpErrorCode == 0 && item._status == SyncFileItem::SoftError) {
+        // assume a local error
+        entry._errorCategory = SyncJournalErrorBlacklistRecord::Category::LocalSoftError;
     }
 
     return entry;
@@ -193,11 +191,9 @@ static void blacklistUpdate(SyncJournalDb *journal, SyncFileItem &item)
 {
     SyncJournalErrorBlacklistRecord oldEntry = journal->errorBlacklistEntry(item._file);
 
-    const bool mayBlacklist = (item._status == SyncFileItem::NormalError)
-        || ((item._status == SyncFileItem::SoftError
-                || item._status == SyncFileItem::DetailError)
-            && item._httpErrorCode != 0 // or non-local error
-        );
+    const bool mayBlacklist = item._status == SyncFileItem::NormalError
+        || item._status == SyncFileItem::SoftError
+        || item._status == SyncFileItem::DetailError;
 
     // No new entry? Possibly remove the old one, then done.
     if (!mayBlacklist) {
@@ -213,24 +209,20 @@ static void blacklistUpdate(SyncJournalDb *journal, SyncFileItem &item)
     // Suppress the error if it was and continues to be blacklisted.
     // An ignoreDuration of 0 mean we're tracking the error, but not actively
     // suppressing it.
-    if (item._hasBlacklistEntry && newEntry._ignoreDuration > 0) {
-        item._status = SyncFileItem::BlacklistedError;
-
-        qCInfo(lcPropagator) << "blacklisting " << item._file
-                             << " for " << newEntry._ignoreDuration
-                             << ", retry count " << newEntry._retryCount;
-
-        return;
-    }
-
     // Some soft errors might become louder on repeat occurrence
     if (item._status == SyncFileItem::SoftError
-        && newEntry._retryCount > 1) {
-        qCWarning(lcPropagator) << "escalating soft error on " << item._file
+        && newEntry._retryCount > 1
+        && item._httpErrorCode != 0) {
+        qCWarning(lcPropagator) << "escalating http soft error on " << item._file
                                 << " to normal error, " << item._httpErrorCode;
         item._status = SyncFileItem::NormalError;
-        return;
+    } else if (item._status != SyncFileItem::SoftError && item._hasBlacklistEntry && newEntry._ignoreDuration > 0) {
+        item._status = SyncFileItem::BlacklistedError;
     }
+
+    qCInfo(lcPropagator) << "blacklisting " << item._file
+                         << " for " << newEntry._ignoreDuration
+                         << ", retry count " << newEntry._retryCount;
 }
 
 void PropagateItemJob::done(SyncFileItem::Status statusArg, const QString &errorString)
@@ -287,15 +279,27 @@ void PropagateItemJob::done(SyncFileItem::Status statusArg, const QString &error
     case SyncFileItem::Excluded:
         // nothing
         break;
+    case SyncFileItem::StatusCount:
+        Q_UNREACHABLE();
     }
 
     if (_item->hasErrorStatus())
         qCWarning(lcPropagator) << "Could not complete propagation of" << _item->destination() << "by" << this << "with status" << _item->_status << "and error:" << _item->_errorString;
     else
         qCInfo(lcPropagator) << "Completed propagation of" << _item->destination() << "by" << this << "with status" << _item->_status;
-    emit propagator()->itemCompleted(_item);
-    emit finished(_item->_status);
 
+    // Will be handled in PropagateDirectory::slotSubJobsFinished at the end
+    if (!_item->isDirectory() || _item->_relevantDirectoyInstruction) {
+        // we are either not a directory or we are a PropagateDirectory job
+        // and an actual instruction was performed for this directory.
+        Q_ASSERT(!_item->_relevantDirectoyInstruction || qobject_cast<PropagateDirectory *>(this));
+        emit propagator()->itemCompleted(_item);
+    } else {
+        // the directoy needs to call done() in PropagateDirectory::slotSubJobsFinished
+        // we don't notify itemCompleted yet as the directory is only complete once its child items are complete.
+        _item->_relevantDirectoyInstruction = true;
+    }
+    emit finished(_item->_status);
     if (_item->_status == SyncFileItem::FatalError) {
         // Abort all remaining jobs.
         propagator()->abort();
@@ -754,6 +758,18 @@ QString OwncloudPropagator::adjustRenamedPath(const QString &original) const
 
 Result<Vfs::ConvertToPlaceholderResult, QString> OwncloudPropagator::updatePlaceholder(const SyncFileItem &item, const QString &fileName, const QString &replacesFile)
 {
+    Q_ASSERT([&] {
+        if (item._type == ItemTypeVirtualFileDehydration) {
+            // when dehydrating the file must not be pinned
+            // don't use destinatio() with suffix placeholder
+            const auto pin = syncOptions()._vfs->pinState(item._file);
+            if (pin && pin.get() == PinState::AlwaysLocal) {
+                qDebug() << fileName << item.destination() << item._file;
+                return false;
+            }
+        }
+        return true;
+    }());
     return syncOptions()._vfs->updateMetadata(item, fileName, replacesFile);
 }
 
@@ -990,39 +1006,62 @@ void PropagateDirectory::slotFirstJobFinished(SyncFileItem::Status status)
 
 void PropagateDirectory::slotSubJobsFinished(const SyncFileItem::Status status)
 {
-    if (!_item->isEmpty() && status == SyncFileItem::Success) {
-        // If a directory is renamed, recursively delete any stale items
-        // that may still exist below the old path.
-        if (_item->_instruction == CSYNC_INSTRUCTION_RENAME
-            && _item->_originalFile != _item->_renameTarget) {
-            propagator()->_journal->deleteFileRecord(_item->_originalFile, true);
+    if (OC_ENSURE(!_item->isEmpty())) {
+        // report an error if the acutal action on the folder failed
+        if (_item->_relevantDirectoyInstruction && _item->_status != SyncFileItem::Success) {
+            Q_ASSERT(_item->_status != SyncFileItem::NoStatus);
+            qCWarning(lcDirectory) << "PropagateDirectory completed with" << status << "the dirctory job itself is marked as" << _item->_status;
+            done(_item->_status);
+            return;
         }
 
-        if (_item->_instruction == CSYNC_INSTRUCTION_NEW && _item->_direction == SyncFileItem::Down) {
-            // special case for local MKDIR, set local directory mtime
-            // (it's not synced later at all, but can be nice to have it set initially)
-            FileSystem::setModTime(propagator()->fullLocalPath(_item->destination()), _item->_modtime);
-        }
+        if (status == SyncFileItem::Success) {
+            // If a directory is renamed, recursively delete any stale items
+            // that may still exist below the old path.
+            if (_item->_instruction == CSYNC_INSTRUCTION_RENAME
+                && _item->_originalFile != _item->_renameTarget) {
+                // TODO: check result but it breaks TestDatabaseError
+                propagator()->_journal->deleteFileRecord(_item->_originalFile, true);
+            }
 
-        // For new directories we always want to update the etag once
-        // the directory has been propagated. Otherwise the directory
-        // could appear locally without being added to the database.
-        if (_item->_instruction & (CSYNC_INSTRUCTION_RENAME | CSYNC_INSTRUCTION_NEW | CSYNC_INSTRUCTION_UPDATE_METADATA)) {
-            const auto result = propagator()->updateMetadata(*_item);
-            if (!result) {
-                qCWarning(lcDirectory) << "Error writing to the database for file" << _item->_file << "with" << result.error();
-                done(SyncFileItem::FatalError, tr("Error updating metadata: %1").arg(result.error()));
-                return;
-            } else if (result.get() == Vfs::ConvertToPlaceholderResult::Locked) {
-                done(SyncFileItem::SoftError, tr("%1 the folder is currently in use").arg(_item->destination()));
+            if (_item->_instruction == CSYNC_INSTRUCTION_NEW && _item->_direction == SyncFileItem::Down) {
+                // special case for local MKDIR, set local directory mtime
+                // (it's not synced later at all, but can be nice to have it set initially)
+                OC_ASSERT(FileSystem::setModTime(propagator()->fullLocalPath(_item->destination()), _item->_modtime));
+            }
+            // For new directories we always want to update the etag once
+            // the directory has been propagated. Otherwise the directory
+            // could appear locally without being added to the database.
+            // Additionally we need to convert those folders to placeholders with cfapi vfs.
+            if (_item->_instruction & (CSYNC_INSTRUCTION_RENAME | CSYNC_INSTRUCTION_NEW | CSYNC_INSTRUCTION_UPDATE_METADATA)) {
+                // metatdata changes are relevant
+                _item->_relevantDirectoyInstruction = true;
+                _item->_status = SyncFileItem::Success;
+                const auto result = propagator()->updateMetadata(*_item);
+                if (!result) {
+                    qCWarning(lcDirectory) << "Error writing to the database for file" << _item->_file << "with" << result.error();
+                    done(SyncFileItem::FatalError, tr("Error updating metadata: %1").arg(result.error()));
+                    return;
+                } else if (result.get() == Vfs::ConvertToPlaceholderResult::Locked) {
+                    done(SyncFileItem::SoftError, tr("%1 the folder is currently in use").arg(_item->destination()));
+                    return;
+                }
+            }
+            if (_item->_relevantDirectoyInstruction) {
+                done(_item->_status);
                 return;
             }
         }
     }
+
     // don't call done, we only propagate the state of the child items
+    // and we don't want error handling for this folder for an error that happend on a child
     Q_ASSERT(_state != Finished);
     _state = Finished;
     emit finished(status);
+    if (_item->_relevantDirectoyInstruction) {
+        emit propagator()->itemCompleted(_item);
+    }
 }
 
 PropagateRootDirectory::PropagateRootDirectory(OwncloudPropagator *propagator)
