@@ -84,6 +84,16 @@ State machine:
 
 void PropagateUploadFileNG::doStartUpload()
 {
+    const QString fileName = propagator()->fullLocalPath(_item->_file);
+    // If the file is currently locked, we want to retry the sync
+    // when it becomes available again.
+    const auto lockMode = propagator()->syncOptions().requiredLockMode();
+    if (FileSystem::isFileLocked(fileName, lockMode)) {
+        emit propagator()->seenLockedFile(fileName, lockMode);
+        abortWithError(SyncFileItem::SoftError, tr("%1 the file is currently in use").arg(QDir::toNativeSeparators(fileName)));
+        return;
+    }
+
     propagator()->_activeJobList.append(this);
 
     UploadRangeInfo rangeinfo = { 0, _item->_size };
@@ -166,8 +176,11 @@ void PropagateUploadFileNG::slotPropfindFinished()
     _currentChunkOffset = 0;
     _sent = 0;
 
-    for (auto chunkOffset : _serverChunks.keys()) {
-        auto chunkSize = _serverChunks[chunkOffset].size;
+    // here is a copy because we might need to remove item(s) during iteration
+    const auto serverChunks = _serverChunks;
+    for (auto it = serverChunks.cbegin(); it != serverChunks.cend(); ++it) {
+        const auto &chunkOffset = it.key();
+        const auto &chunkSize = it.value().size;
         if (markRangeAsDone(chunkOffset, chunkSize)) {
             qCDebug(lcPropagateUploadNG) << "Reusing existing data:" << chunkOffset << chunkSize;
             _sent += chunkSize;
@@ -255,7 +268,7 @@ void PropagateUploadFileNG::slotDeleteJobFinished()
 
     // If no more Delete jobs are running, we can continue
     bool runningDeleteJobs = false;
-    for (auto otherJob : _jobs) {
+    for (auto *otherJob : qAsConst(_jobs)) {
         if (qobject_cast<DeleteJob *>(otherJob))
             runningDeleteJobs = true;
     }
@@ -290,13 +303,15 @@ void PropagateUploadFileNG::startNewUpload()
     QMap<QByteArray, QByteArray> headers;
     headers["OC-Total-Length"] = QByteArray::number(_item->_size);
     auto job = new MkColJob(propagator()->account(), chunkUrl(), headers, this);
-    connect(job, SIGNAL(finished(QNetworkReply::NetworkError)),
-        this, SLOT(slotMkColFinished(QNetworkReply::NetworkError)));
+    connect(job, &MkColJob::finishedWithError,
+        this, &PropagateUploadFileNG::slotMkColFinished);
+    connect(job, &MkColJob::finishedWithoutError,
+        this, &PropagateUploadFileNG::slotMkColFinished);
     connect(job, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
     job->start();
 }
 
-void PropagateUploadFileNG::slotMkColFinished(QNetworkReply::NetworkError)
+void PropagateUploadFileNG::slotMkColFinished()
 {
     propagator()->_activeJobList.removeOne(this);
     auto job = qobject_cast<MkColJob *>(sender());
@@ -370,17 +385,10 @@ void PropagateUploadFileNG::startNextChunk()
     _currentChunkSize = qMin(propagator()->_chunkSize, _rangesToUpload.first().size);
 
     const QString fileName = propagator()->fullLocalPath(_item->_file);
-
     auto device = std::unique_ptr<UploadDevice>(new UploadDevice(
             fileName, _currentChunkOffset, _currentChunkSize, &propagator()->_bandwidthManager));
     if (!device->open(QIODevice::ReadOnly)) {
         qCWarning(lcPropagateUploadNG) << "Could not prepare upload device: " << device->errorString();
-
-        // If the file is currently locked, we want to retry the sync
-        // when it becomes available again.
-        if (FileSystem::isFileLocked(fileName)) {
-            emit propagator()->seenLockedFile(fileName);
-        }
         // Soft error because this is likely caused by the user modifying his files while syncing
         abortWithError(SyncFileItem::SoftError, device->errorString());
         return;
@@ -517,13 +525,7 @@ void PropagateUploadFileNG::slotMoveJobFinished()
     }
 
     if (_item->_httpErrorCode == 202) {
-        QString path = QString::fromUtf8(job->reply()->rawHeader("OC-JobStatus-Location"));
-        if (path.isEmpty()) {
-            done(SyncFileItem::NormalError, tr("Poll URL missing"));
-            return;
-        }
-        _finished = true;
-        startPollJob(path);
+        done(SyncFileItem::NormalError, tr("The server did ask for a removed legacy feature(polling)"));
         return;
     }
 
