@@ -3,11 +3,12 @@ import names
 import os
 import sys
 from os import listdir
-from os.path import isfile, join
+from os.path import isfile, join, isdir
 import re
 import urllib.request
 import json
 import requests
+import shutil
 
 from objectmaphelper import RegularExpression
 from pageObjects.AccountConnectionWizard import AccountConnectionWizard
@@ -63,35 +64,30 @@ def hook(context):
         pass
 
 
-@Given('the user has added an account with')
-def step(context):
-    toolbar = Toolbar()
-    toolbar.clickAddAccount()
-
+@Given(r'the user has added (the first|another) account with', regexp=True)
+def step(context, accountType):
     newAccount = AccountConnectionWizard()
+    if accountType == 'another':
+        toolbar = Toolbar()
+        toolbar.clickAddAccount()
+
     newAccount.addAccount(context)
 
 
-@When('the user adds the first account with')
+@When('the user adds the following wrong user credentials:')
 def step(context):
     newAccount = AccountConnectionWizard()
-    newAccount.addAccount(context)
+    newAccount.addUserCreds(context)
 
 
 @Then('an account should be displayed with the displayname |any| and host |any|')
 def step(context, displayname, host):
     displayname = substituteInLineCodes(context, displayname)
     host = substituteInLineCodes(context, host)
+    accountStatus = AccountStatus(context, displayname, host)
+
     test.compare(
-        str(
-            waitForObjectExists(
-                {
-                    "name": "settingsdialog_toolbutton_" + displayname + "@" + host,
-                    "type": "QToolButton",
-                    "visible": 1,
-                }
-            ).text
-        ),
+        accountStatus.getText(),
         displayname + "\n" + host,
     )
 
@@ -120,15 +116,24 @@ def step(context):
     startClient(context)
 
 
-@When('the user adds an account with')
-def step(context):
-    toolbar = Toolbar()
-    toolbar.clickAddAccount()
-
+@When(r'the user adds (the first|another) account with', regexp=True)
+def step(context, accountType):
     newAccount = AccountConnectionWizard()
+    if accountType == 'another':
+        toolbar = Toolbar()
+        toolbar.clickAddAccount()
+
     newAccount.addAccount(context)
 
 
+@Given('the user has added the following account information:')
+def step(context):
+    newAccount = AccountConnectionWizard()
+    newAccount.addServer(context)
+    newAccount.addUserCreds(context)
+    newAccount.selectSyncFolder(context)
+
+
 def isItemSynced(type, itemName):
     if type != 'FILE' and type != 'FOLDER':
         raise Exception("type must be 'FILE' or 'FOLDER'")
@@ -155,7 +160,7 @@ def isFileSynced(fileName):
 def waitForFileToBeSynced(context, fileName):
     waitFor(
         lambda: isFileSynced(
-            sanitizePath(context.userData['clientSyncPathUser1'] + fileName)
+            sanitizePath(context.userData['currentUserSyncPath'] + fileName)
         ),
         context.userData['clientSyncTimeout'] * 1000,
     )
@@ -164,12 +169,26 @@ def waitForFileToBeSynced(context, fileName):
 def waitForFolderToBeSynced(context, folderName):
     waitFor(
         lambda: isFolderSynced(
-            sanitizePath(context.userData['clientSyncPathUser1'] + folderName)
+            sanitizePath(context.userData['currentUserSyncPath'] + folderName)
         ),
         context.userData['clientSyncTimeout'] * 1000,
     )
 
 
+def folderExists(folderPath, timeout=1000):
+    return waitFor(
+        lambda: isdir(sanitizePath(folderPath)),
+        timeout,
+    )
+
+
+def fileExists(filePath, timeout=1000):
+    return waitFor(
+        lambda: isfile(sanitizePath(filePath)),
+        timeout,
+    )
+
+
 def sanitizePath(path):
     return path.replace('//', '/')
 
@@ -214,6 +233,30 @@ def step(context, receiver, resource, permissions):
     openSharingDialog(context, resource)
     shareItem = SharingDialog()
     shareItem.addCollaborator(receiver, permissions)
+    shareItem.closeSharingDialog()
+
+
+@When('the user adds following collaborators of resource "|any|" using the client-UI')
+def step(context, resource):
+    openSharingDialog(context, resource)
+    shareItem = SharingDialog()
+
+    # In the following loop we are trying to share resource with given permission to one user at a time given from the data table in the feature file
+    for row in context.table[1:]:
+        receiver = row[0]
+        permissions = row[1]
+        shareItem.addCollaborator(receiver, permissions)
+
+    shareItem.closeSharingDialog()
+
+
+@When(
+    'the user selects "|any|" as collaborator of resource "|any|" using the client-UI'
+)
+def step(context, receiver, resource):
+    openSharingDialog(context, resource)
+    shareItem = SharingDialog()
+    shareItem.selectCollaborator(receiver)
 
 
 @When(
@@ -223,6 +266,7 @@ def step(context, receiver, resource, permissions):
     openSharingDialog(context, resource)
     shareItem = SharingDialog()
     shareItem.addCollaborator(receiver, permissions, True)
+    shareItem.closeSharingDialog()
 
 
 @Then(
@@ -240,15 +284,39 @@ def step(context, receiver, resource, permissions):
     collaboratorShouldBeListed(context, receiver, resource, permissions)
 
 
-def collaboratorShouldBeListed(context, receiver, resource, permissions):
-    resource = substituteInLineCodes(context, resource)
+def collaboratorShouldBeListed(
+    context, receiver, resource, permissions, receiverCount=0
+):
+    resource = getResourcePath(context, resource)
     socketConnect = syncstate.SocketConnect()
     socketConnect.sendCommand("SHARE:" + resource + "\n")
     permissionsList = permissions.split(',')
 
-    test.compare(
-        str(waitForObjectExists(names.scrollArea_sharedWith_QLabel).text), receiver
+    waitForObject(
+        {
+            "container": names.sharingDialogUG_scrollArea_QScrollArea,
+            "name": "sharedWith",
+            "type": "QLabel",
+            "visible": 1,
+        }
     )
+
+    # findAllObjects: This functionfinds and returns a list of object references identified by the symbolic or real (multi-property) name objectName.
+    sharedWithObj = findAllObjects(
+        {
+            "container": names.sharingDialogUG_scrollArea_QScrollArea,
+            "name": "sharedWith",
+            "type": "QLabel",
+            "visible": 1,
+        }
+    )
+
+    #     we use sharedWithObj list from above while verifying if users are listed or not.
+    #     For this we need an index value i.e receiverCount
+    #     For 1st user in the list the index will be 0 which is receiverCount default value
+    #     For 2nd user in the list the index will be 1 and so on
+
+    test.compare(str(sharedWithObj[receiverCount].text), receiver)
     test.compare(
         waitForObjectExists(names.scrollArea_permissionsEdit_QCheckBox).checked,
         ('edit' in permissionsList),
@@ -257,6 +325,8 @@ def collaboratorShouldBeListed(context, receiver, resource, permissions):
         waitForObjectExists(names.scrollArea_permissionShare_QCheckBox).checked,
         ('share' in permissionsList),
     )
+    shareItem = SharingDialog()
+    shareItem.closeSharingDialog()
 
 
 @When('the user waits for the files to sync')
@@ -269,19 +339,69 @@ def step(context, fileName):
     waitForFileToBeSynced(context, fileName)
 
 
+@When('the user waits for folder "|any|" to be synced')
+def step(context, folderName):
+    waitForFolderToBeSynced(context, folderName)
+
+
 @Given('the user has waited for file "|any|" to be synced')
 def step(context, fileName):
     waitForFileToBeSynced(context, fileName)
 
 
-@When('the user creates a file "|any|" with the following content on the file system')
-def step(context, filename):
+@Given(
+    'user "|any|" has created a file "|any|" with the following content inside the sync folder'
+)
+def step(context, username, filename):
+    createFile(context, filename, username)
+
+
+@When(
+    'user "|any|" creates a file "|any|" with the following content inside the sync folder'
+)
+def step(context, username, filename):
+    createFile(context, filename, username)
+
+
+def createFile(context, filename, username=None):
     fileContent = "\n".join(context.multiLineText)
-    f = open(context.userData['clientSyncPathUser1'] + filename, "w")
+    syncPath = None
+    if username:
+        syncPath = getUserSyncPath(context, username)
+    else:
+        syncPath = context.userData['currentUserSyncPath']
+    f = open(join(syncPath, filename), "w")
     f.write(fileContent)
     f.close()
 
 
+@When('user "|any|" creates a folder "|any|" inside the sync folder')
+def step(context, username, foldername):
+    createFolder(context, foldername, username)
+
+
+@Given('user "|any|" has created a folder "|any|" inside the sync folder')
+def step(context, username, foldername):
+    createFolder(context, foldername, username)
+
+
+def createFolder(context, foldername, username=None):
+    syncPath = None
+    if username:
+        syncPath = getUserSyncPath(context, username)
+    else:
+        syncPath = context.userData['currentUserSyncPath']
+    path = join(syncPath, foldername)
+    os.makedirs(path)
+
+
+@When('the user copies the folder "|any|" to "|any|"')
+def step(context, sourceFolder, destinationFolder):
+    source_dir = join(context.userData['currentUserSyncPath'], sourceFolder)
+    destination_dir = join(context.userData['currentUserSyncPath'], destinationFolder)
+    shutil.copytree(source_dir, destination_dir)
+
+
 @Given(r"^(.*) on the server (.*)$", regexp=True)
 def step(context, stepPart1, stepPart2):
     executeStepThroughMiddleware(context, "Given " + stepPart1 + " " + stepPart2)
@@ -309,7 +429,7 @@ def step(context, stepPart1):
 @Then('the file "|any|" should exist on the file system with the following content')
 def step(context, filePath):
     expected = "\n".join(context.multiLineText)
-    filePath = context.userData['clientSyncPathUser1'] + filePath
+    filePath = context.userData['currentUserSyncPath'] + filePath
     f = open(filePath, 'r')
     contents = f.read()
     test.compare(
@@ -321,6 +441,46 @@ def step(context, filePath):
     )
 
 
+@Then(r'^the (file|folder) "([^"]*)" should exist on the file system$', regexp=True)
+def step(context, resourceType, resource):
+    resourcePath = join(context.userData['currentUserSyncPath'], resource)
+    resourceExists = False
+    if resourceType == 'file':
+        resourceExists = fileExists(
+            resourcePath, context.userData['clientSyncTimeout'] * 1000
+        )
+    elif resourceType == 'folder':
+        resourceExists = folderExists(
+            resourcePath, context.userData['clientSyncTimeout'] * 1000
+        )
+    else:
+        raise Exception("Unsupported resource type '" + resourceType + "'")
+
+    test.compare(
+        True,
+        resourceExists,
+        "Assert " + resourceType + " '" + resource + "' exists on the system",
+    )
+
+
+@Then(r'^the (file|folder) "([^"]*)" should not exist on the file system$', regexp=True)
+def step(context, resourceType, resource):
+    resourcePath = join(context.userData['currentUserSyncPath'], resource)
+    resourceExists = False
+    if resourceType == 'file':
+        resourceExists = fileExists(resourcePath, 1000)
+    elif resourceType == 'folder':
+        resourceExists = folderExists(resourcePath, 1000)
+    else:
+        raise Exception("Unsupported resource type '" + resourceType + "'")
+
+    test.compare(
+        False,
+        resourceExists,
+        "Assert " + resourceType + " '" + resource + "' doesn't exist on the system",
+    )
+
+
 @Given('the user has paused the file sync')
 def step(context):
     waitForFolderToBeSynced(context, '/')
@@ -331,7 +491,7 @@ def step(context):
 @Given('the user has changed the content of local file "|any|" to:')
 def step(context, filename):
     fileContent = "\n".join(context.multiLineText)
-    f = open(context.userData['clientSyncPathUser1'] + filename, "w")
+    f = open(context.userData['currentUserSyncPath'] + filename, "w")
     f.write(fileContent)
     f.close()
 
@@ -364,14 +524,14 @@ def step(context, filename):
     extpart = filename.split('.')[1]
     onlyfiles = [
         f
-        for f in listdir(context.userData['clientSyncPathUser1'])
-        if isfile(join(context.userData['clientSyncPathUser1'], f))
+        for f in listdir(context.userData['currentUserSyncPath'])
+        if isfile(join(context.userData['currentUserSyncPath'], f))
     ]
     found = False
     pattern = re.compile(buildConflictedRegex(filename))
     for file in onlyfiles:
         if pattern.match(file):
-            f = open(context.userData['clientSyncPathUser1'] + file, 'r')
+            f = open(context.userData['currentUserSyncPath'] + file, 'r')
             contents = f.read()
             if contents == expected:
                 found = True
@@ -413,6 +573,12 @@ def step(context, filename):
     activity.checkFileExist(filename)
 
 
+@Then('the file "|any|" should be blacklisted')
+def step(context, filename):
+    activity = Activity()
+    activity.checkBlackListedFileExist(filename)
+
+
 @When('the user selects "|any|" tab in the activity')
 def step(context, tabName):
     activity = Activity()
@@ -420,7 +586,7 @@ def step(context, tabName):
 
 
 def openSharingDialog(context, resource, itemType='file'):
-    resource = sanitizePath(substituteInLineCodes(context, resource))
+    resource = getResourcePath(context, resource)
 
     if itemType == 'folder':
         waitFor(
@@ -441,7 +607,6 @@ def openSharingDialog(context, resource, itemType='file'):
 
 @When('the user opens the public links dialog of "|any|" using the client-UI')
 def step(context, resource):
-    resource = sanitizePath(substituteInLineCodes(context, resource))
     openSharingDialog(context, resource)
     publicLinkDialog = PublicLinkDialog()
     publicLinkDialog.openPublicLinkDialog()
@@ -450,7 +615,7 @@ def step(context, resource):
 @When("the user toggles the password protection using the client-UI")
 def step(context):
     publicLinkDialog = PublicLinkDialog()
-    publicLinkDialog.togglesPassword()
+    publicLinkDialog.togglePassword()
 
 
 @Then('the password progress indicator should not be visible in the client-UI')
@@ -465,27 +630,45 @@ def step(context):
     waitFor(lambda: (test.xvp("publicLinkPasswordProgressIndicatorInvisible")))
 
 
-@When('user "|any|" opens the sharing dialog of "|any|" using the client-UI')
-def step(context, receiver, resource):
+@When('the user opens the sharing dialog of "|any|" using the client-UI')
+def step(context, resource):
     openSharingDialog(context, resource, 'folder')
 
 
-@Then('the error text "|any|" should be displayed in the sharing dialog')
-def step(context, fileShareContext):
+def getSharingDialogText():
     shareItem = SharingDialog()
     errorText = shareItem.getSharingDialogMessage()
+    return errorText
+
+
+@Then('the text "|any|" should be displayed in the sharing dialog')
+def step(context, fileShareContext):
+    errorText = getSharingDialogText()
     test.compare(
         errorText,
         fileShareContext,
     )
 
 
-def createPublicLinkShare(context, resource, password='', permissions=''):
-    resource = sanitizePath(substituteInLineCodes(context, resource))
+@Then('the error text "|any|" should be displayed in the sharing dialog')
+def step(context, fileShareContext):
+    errorText = getSharingDialogText()
+    test.compare(
+        errorText,
+        fileShareContext,
+    )
+
+
+def createPublicLinkShare(
+    context, resource, password='', permissions='', expireDate='', name=''
+):
+    resource = getResourcePath(context, resource)
     openSharingDialog(context, resource)
     publicLinkDialog = PublicLinkDialog()
     publicLinkDialog.openPublicLinkDialog()
-    publicLinkDialog.createPublicLink(context, resource, password, permissions)
+    publicLinkDialog.createPublicLink(
+        context, resource, password, permissions, expireDate, name
+    )
 
 
 @When(
@@ -502,10 +685,21 @@ def step(context, resource, password):
     createPublicLinkShare(context, resource, password)
 
 
+def setExpirationDateWithVerification(resource, publicLinkName, expireDate):
+    publicLinkDialog = PublicLinkDialog()
+    publicLinkDialog.verifyResource(resource)
+    publicLinkDialog.verifyPublicLinkName(publicLinkName)
+    publicLinkDialog.setExpirationDate(expireDate)
+
+
 @When('the user edits the public link named "|any|" of file "|any|" changing following')
 def step(context, publicLinkName, resource):
-    publicLinkDialog = PublicLinkDialog()
-    publicLinkDialog.setExpirationDate(context, publicLinkName, resource)
+    expireDate = ''
+    for row in context.table:
+        if row[0] == 'expireDate':
+            expireDate = row[1]
+            break
+    setExpirationDateWithVerification(resource, publicLinkName, expireDate)
 
 
 @When(
@@ -522,6 +716,19 @@ def step(context, permissions, resource, password):
     createPublicLinkShare(context, resource, password, permissions)
 
 
+@When('the user creates a new public link with following settings using the client-UI:')
+def step(context):
+    linkSettings = {}
+    for row in context.table:
+        linkSettings[row[0]] = row[1]
+    createPublicLinkShare(
+        context,
+        resource=linkSettings['path'],
+        password=linkSettings['password'],
+        expireDate=linkSettings['expireDate'],
+    )
+
+
 def createPublicShareWithRole(context, resource, role):
     resource = sanitizePath(substituteInLineCodes(context, resource))
     openSharingDialog(context, resource)
@@ -546,23 +753,16 @@ def step(context, resource):
         createPublicShareWithRole(context, resource, role)
 
 
-@When(
-    'the user creates a new public link for folder "|any|" with "|any|" using the client-UI'
-)
-def step(context, resource, role):
-    createPublicShareWithRole(context, resource, role)
-
-
-@When('the user logs out of the client-UI')
-def step(context):
-    accountStatus = AccountStatus()
+@When('the user "|any|" logs out of the client-UI')
+def step(context, username):
+    accountStatus = AccountStatus(context, getDisplaynameForUser(context, username))
     accountStatus.accountAction("Log out")
 
 
 def isUserSignedOut(context, username):
     displayname = getDisplaynameForUser(context, username)
     server = context.userData['localBackendUrl']
-    accountStatus = AccountStatus()
+    accountStatus = AccountStatus(context, getDisplaynameForUser(context, username))
     test.compare(
         str(waitForObjectExists(accountStatus.SIGNED_OUT_TEXT_BAR).text),
         'Signed out from <a href="'
@@ -578,8 +778,7 @@ def isUserSignedOut(context, username):
 def isUserSignedIn(context, username):
     displayname = getDisplaynameForUser(context, username)
     server = context.userData['localBackendUrl']
-    accountStatus = AccountStatus()
-
+    accountStatus = AccountStatus(context, getDisplaynameForUser(context, username))
     test.compare(
         str(waitForObjectExists(accountStatus.SIGNED_OUT_TEXT_BAR).text),
         'Connected '
@@ -604,14 +803,14 @@ def step(context, username):
     # TODO: find some way to dynamically to check if files are synced
     # It might take some time for all files to sync
     snooze(5)
-    accountStatus = AccountStatus()
+    accountStatus = AccountStatus(context, getDisplaynameForUser(context, username))
     accountStatus.accountAction("Log out")
     isUserSignedOut(context, username)
 
 
 @When('user "|any|" logs in to the client-UI')
 def step(context, username):
-    accountStatus = AccountStatus()
+    accountStatus = AccountStatus(context, getDisplaynameForUser(context, username))
     accountStatus.accountAction("Log in")
     password = getPasswordForUser(context, username)
     enterUserPassword = EnterPassword()
@@ -632,18 +831,8 @@ def step(context, username, host):
     displayname = substituteInLineCodes(context, displayname)
     host = substituteInLineCodes(context, host)
 
-    clickButton(
-        waitForObject(
-            {
-                "name": "settingsdialog_toolbutton_" + displayname + "@" + host,
-                "type": "QToolButton",
-                "visible": 1,
-            }
-        )
-    )
-
     waitForFolderToBeSynced(context, '/')
-    accountStatus = AccountStatus()
+    accountStatus = AccountStatus(context, displayname, host)
     accountStatus.removeConnection()
 
 
@@ -700,6 +889,7 @@ def step(context, permissions, user, resource):
     permissionsList = permissions.split(',')
 
     shareItem = SharingDialog()
+    shareItem.verifyResource(resource)
     editChecked, shareChecked = shareItem.getAvailablePermission()
 
     if 'edit' in permissionsList:
@@ -738,7 +928,7 @@ def step(context, resource, content):
 
     snooze(5)
 
-    f = open(context.userData['clientSyncPathUser1'] + resource, "w")
+    f = open(context.userData['currentUserSyncPath'] + resource, "w")
     f.write(content)
     f.close()
 
@@ -746,6 +936,21 @@ def step(context, resource, content):
     waitForFileToBeSynced(context, resource)
 
 
+@When('the user tries to overwrite the file "|any|" with content "|any|"')
+def step(context, resource, content):
+    waitForFileToBeSynced(context, resource)
+    waitForFolderToBeSynced(context, '/')
+
+    try:
+        f = open(context.userData['currentUserSyncPath'] + resource, "w")
+        f.write(content)
+        f.close()
+    except:
+        pass
+
+    waitForFileToBeSynced(context, resource)
+
+
 def enableVFSSupport(vfsBtnText):
     # The enabling/disabling VFS button do not have it's own object
     # But it is inside the "stack_folderList_QTreeView" object.
@@ -806,3 +1011,257 @@ def step(context):
     clickButton(
         waitForObject(names.disable_virtual_file_support_Disable_support_QPushButton)
     )
+
+
+@When('the user accepts the certificate')
+def step(context):
+    clickButton(waitForObject(names.oCC_SslErrorDialog_cbTrustConnect_QCheckBox))
+    clickButton(waitForObject(names.oCC_SslErrorDialog_OK_QPushButton))
+
+
+@Then('the lock shown should be closed')
+def step(context):
+    test.vp("urlLock")
+
+
+@Then('error "|any|" should be displayed')
+def step(context, errorMsg):
+    newAccount = AccountConnectionWizard()
+    test.compare(str(waitForObjectExists(newAccount.ERROR_LABEL).text), errorMsg)
+
+
+@When(r'the user deletes the (file|folder) "([^"]*)"', regexp=True)
+def step(context, itemType, resource):
+    resourcePath = sanitizePath(context.userData['currentUserSyncPath'] + resource)
+    if itemType == 'file':
+        os.remove(resourcePath)
+    elif itemType == 'folder':
+        shutil.rmtree(resourcePath)
+    else:
+        raise Exception("No such item type for resource")
+
+
+@When(
+    'the user unshares the resource "|any|" for collaborator "|any|" using the client-UI'
+)
+def step(context, resource, receiver):
+    openSharingDialog(context, resource)
+    test.compare(
+        str(waitForObjectExists(names.scrollArea_sharedWith_QLabel).text), receiver
+    )
+    clickButton(waitForObject(names.scrollArea_deleteShareButton_QToolButton))
+
+
+@Given('the user has added the following server address:')
+def step(context):
+    newAccount = AccountConnectionWizard()
+    newAccount.addServer(context)
+    test.compare(
+        waitForObjectExists(newAccount.CREDENTIAL_PAGE).visible,
+        True,
+        "Assert credentials page is visible",
+    )
+
+
+@When('the user adds the following server address:')
+def step(context):
+    newAccount = AccountConnectionWizard()
+    newAccount.addServer(context)
+
+
+@Given('the user has added the following user credentials:')
+def step(context):
+    newAccount = AccountConnectionWizard()
+    newAccount.addUserCreds(context)
+    test.compare(
+        waitForObjectExists(newAccount.ADVANCE_SETUP_PAGE).visible,
+        True,
+        "Assert setup page is visible",
+    )
+
+
+@Given('the user has opened chose_what_to_sync dialog')
+def step(context):
+    newAccount = AccountConnectionWizard()
+    newAccount.openSyncDialog()
+    test.compare(
+        waitForObjectExists(newAccount.SELECTIVE_SYNC_DIALOG).visible,
+        True,
+        "Assert selective sync dialog is visible",
+    )
+
+
+@When('the user opens chose_what_to_sync dialog')
+def step(context):
+    newAccount = AccountConnectionWizard()
+    newAccount.openSyncDialog()
+
+
+@When('the user selects the following folders to sync:')
+def step(context):
+    newAccount = AccountConnectionWizard()
+    newAccount.selectFoldersToSync(context)
+
+
+@When('the user selects manual sync folder option')
+def step(context):
+    newAccount = AccountConnectionWizard()
+    newAccount.selectManualSyncFolder()
+
+
+@When('the user connects the account')
+def step(context):
+    newAccount = AccountConnectionWizard()
+    newAccount.connectAccount()
+
+
+@When('the user sorts the folder list by "|any|"')
+def step(context, headerText):
+    headerText = headerText.capitalize()
+    if headerText in ["Size", "Name"]:
+        newAccount = AccountConnectionWizard()
+        newAccount.sortBy(headerText)
+    else:
+        raise Exception("Sorting by '" + headerText + "' is not supported.")
+
+
+@Then('the dialog chose_what_to_sync should be visible')
+def step(context):
+    newAccount = AccountConnectionWizard()
+    test.compare(
+        waitForObjectExists(newAccount.SELECTIVE_SYNC_DIALOG).visible,
+        True,
+        "Assert selective sync dialog is visible",
+    )
+
+
+@Then('the sync all checkbox should be checked')
+def step(context):
+    newAccount = AccountConnectionWizard()
+    test.compare(
+        waitForObjectExists(newAccount.SYNC_DIALOG_ROOT_FOLDER).checkState,
+        "checked",
+        "Assert sync all checkbox is checked",
+    )
+
+
+@Then("the folders should be in the following order:")
+def step(context):
+    newAccount = AccountConnectionWizard()
+    rowIndex = 0
+    for row in context.table[1:]:
+        FOLDER_TREE_ROW = {
+            "row": rowIndex,
+            "container": newAccount.SYNC_DIALOG_ROOT_FOLDER,
+            "type": "QModelIndex",
+        }
+        expectedFolder = row[0]
+        actualFolder = waitForObjectExists(FOLDER_TREE_ROW).displayText
+        test.compare(actualFolder, expectedFolder)
+
+        rowIndex += 1
+
+
+@When('the user deletes the public link for file "|any|"')
+def step(context, resource):
+    openSharingDialog(context, resource)
+    publicLinkDialog = PublicLinkDialog()
+    publicLinkDialog.openPublicLinkDialog()
+
+    test.compare(
+        str(waitForObjectExists(publicLinkDialog.ITEM_TO_SHARE).text),
+        resource.replace(context.userData['currentUserSyncPath'], ''),
+    )
+    clickButton(waitForObject(names.linkShares_QToolButton_2))
+    clickButton(waitForObject(names.oCC_ShareLinkWidget_Delete_QPushButton))
+
+    waitFor(
+        lambda: (not object.exists(names.linkShares_QToolButton_2)),
+    )
+
+
+@When(
+    'the user changes the password of public link "|any|" to "|any|" using the client-UI'
+)
+def step(context, publicLinkName, password):
+    publicLinkDialog = PublicLinkDialog()
+    publicLinkDialog.verifyPublicLinkName(publicLinkName)
+    publicLinkDialog.changePassword(password)
+
+
+@Then(
+    'the following users should be listed in as collaborators for file "|any|" on the client-UI'
+)
+def step(context, resource):
+    #     Here we are trying to verify if the user added in when step are listed in the client-UI or not
+    #     We now have a variable name receiverCount which is used in collaboratorShouldBeListed function call
+    receiverCount = 0
+    for row in context.table[1:]:
+        receiver = row[0]
+        permissions = row[1]
+
+        collaboratorShouldBeListed(
+            context, receiver, resource, permissions, receiverCount
+        )
+        receiverCount += 1
+
+
+def searchCollaborator(collaborator):
+    shareItem = SharingDialog()
+    shareItem.searchCollaborator(collaborator)
+
+
+@When('the user searches for collaborator "|any|" using the client-UI')
+def step(context, collaborator):
+    searchCollaborator(collaborator)
+
+
+@When(
+    'the user searches for collaborator with autocomplete characters "|any|" using the client-UI'
+)
+def step(context, collaborator):
+    searchCollaborator(collaborator)
+
+
+@Then('the following users should be listed as suggested collaborators:')
+def step(context):
+    shareItem = SharingDialog()
+    for collaborator in context.table[1:]:
+        exists = False
+        try:
+            waitForObjectItem(shareItem.SUGGESTED_COLLABORATOR, collaborator[0])
+            exists = True
+        except LookupError as e:
+            pass
+
+        test.compare(exists, True, "Assert user '" + collaborator[0] + "' is listed")
+
+
+@Then('the collaborators should be listed in the following order:')
+def step(context):
+    shareItem = SharingDialog()
+    for index, collaborator in enumerate(context.table[1:], start=1):
+        test.compare(
+            str(
+                waitForObjectExists(
+                    {
+                        "container": names.sharingDialogUG_scrollArea_QScrollArea,
+                        "name": "sharedWith",
+                        "occurrence": index,
+                        "type": "QLabel",
+                        "visible": 1,
+                    }
+                ).text
+            ),
+            collaborator[0],
+        )
+
+
+@Then('VFS enabled baseline image should match the default screenshot')
+def step(context):
+    test.vp("VP_VFS_enabled")
+
+
+@Then('VFS enabled baseline image should not match the default screenshot')
+def step(context):
+    test.xvp("VP_VFS_enabled")
